#ifndef __stdio_device_H
#define __stdio_device_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: StdioDevice.h,v 1.3 1996/10/28 03:45:07 steve Exp $"
/*
 * $Log: StdioDevice.h,v $
 * Revision 1.3  1996/10/28 03:45:07  steve
 *  License notice.
 *
 * Revision 1.2  1996/09/19 01:38:30  steve
 *  Add fgetc and fgets. stdio input works.
 *
 * Revision 1.1  1996/09/14 00:10:13  steve
 *  Add a stdio library.
 *
 */

/*
 * These classes are intended as back-end support for the stdio and/or
 * streamio routines. They provide a way of attaching names to device
 * drivers, and providing a standard interface that can be used for
 * manipulating the device without knowing what it is. A device is not
 * required to support these classes, but stdio to the device will be
 * impossible otherwise.
 */

# include  <stddef.h>
class StdioDevice;
class StdioDeviceType;

/*
 * The DeviceType is a factory for Device objects. Each device type
 * has a name that the find method uses to select a DeviceType. The
 * constructor puts the DeviceType into a list (a tree, actually) that
 * find uses for searches.
 *
 * Once the program gets ahold of the desired DeviceType, it can use
 * the open method to get a Device object, which can perform the
 * actual I/O.
 */

class StdioDeviceType {

    public:
      StdioDeviceType(const char*name);
      virtual ~StdioDeviceType();

      virtual StdioDevice*open(const char*filename) =0;
      virtual void close(StdioDevice*) =0;

      static StdioDeviceType* find(const char*name);

    private:
      char name_[8];

      static StdioDeviceType*list_;
      StdioDeviceType*left_, *right_;
};

/*
 * The Device is implicitly opened by creating an instance of this
 * type, and is closed by deleting the object. Normally, Device
 * types are closely associated with DeviceType objects that create
 * them, but that is not a matter for these classes.
 *
 * This is more like a file descriptor then a Device, but oh well.
 */
class StdioDevice {

    public:
      StdioDevice() : flags_(READABLE|WRITEABLE) { }
      virtual ~StdioDevice() =0;

	// Read is used to fill a buffer with input characters. The
	// caller passes the size of the buffer and read returns 0 or
	// more. (0 means EOF.) read returns the number of characters
	// read, or <0 for an error.
      virtual int  read(void*str, size_t cnt) =0;

	// Write puts ALL the characters into the device, blocking if
	// necessary. (The real device may buffer them.)
      virtual void write(const void*str, size_t cnt) =0;

	// The stdio library uses these flags to select default
	// behavior for the device.

      bool interactive() const { return flags_&INTERACTIVE; }
      bool readable()    const { return flags_&READABLE; }
      bool writeable()   const { return flags_&WRITEABLE; }

    protected:
      enum {
	    INTERACTIVE = 0x01,
	    READABLE    = 0x02,
	    WRITEABLE   = 0x04
      };

      unsigned flags_;

    private:
};


#endif
