#ifndef __chips_BIOS32_H
#define __chips_BIOS32_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BIOS32.h,v 1.10 1997/11/03 23:25:40 steve Exp $"


/*
 * This class provides an interface to the PCI bus that one expects of
 * a BIOS32 system. It goes the extra mile too and locates and maps
 * the devices, if that is required.
 */

class Region;

class BIOS32 {

    public:
      static void reset_bus();

    public:
      explicit BIOS32(unsigned long mask);

	// Find a PCI device by vendir_id and device_id. Find the
	// specified instance of the device (there may be more then
	// one of each kind) and return the bus and dev_fn cookies for
	// that device. Return true if the device exists, false
	// otherwise.
      bool find_device(unsigned short vendor_id,
		       unsigned short device_id,
		       unsigned inst,
		       unsigned char&bus, unsigned char&dev_fn) const;


	// Access the configuration registers of a device. All of
	// these functions return false if there was an error
	// accessing the register.

      bool read_config_dword(unsigned char bus, unsigned char dev_fn,
			     unsigned char reg_id, unsigned long&out) const;

      bool write_config_dword(unsigned char bus, unsigned char dev_fn,
			      unsigned char reg_id, unsigned long value) const;

      bool read_config_word(unsigned char bus, unsigned char dev_fn,
			    unsigned char reg_id, unsigned short&out) const;

      bool write_config_word(unsigned char bus, unsigned char dev_fn,
			     unsigned char reg_id, unsigned short value) const;

    public:
	// These methods provide a way to look at all the devices on
	// the bus. One can also use these to make an iterator class.
      unsigned peek_ndevs() const
      { return ndevs_; }

      unsigned long peek_id(unsigned idx) const
      { return devs_[idx].id; }

      void peek_bus_dev_fn(unsigned idx,
			   unsigned char&bus, unsigned char&dev_fn) const
      { bus = devs_[idx].bus; dev_fn = devs_[idx].dev_fn; }

    private:
      unsigned char my_bus;
      bool map_vga_;

      enum { DEV_LIMIT_ = 16 };
      struct DEV_ {
	    unsigned long id;
	    unsigned char bus, dev_fn;
	    bool is_bridge;
      } devs_[DEV_LIMIT_];
      unsigned ndevs_;

      struct SCRATCH_ {
	    unsigned long nmem[6]; // Keep track of memory regions
	    unsigned long pmem[6]; // ... and prefetchable memory
	    unsigned long imem[6]; // ... and IO space
	    unsigned long nunder; 
	    unsigned long punder;
	    unsigned long iunder;
      };

      void probe_bus_(SCRATCH_*scr, unsigned char bus,
		      unsigned long mask, SCRATCH_*bridge =0);

      void map_bus_(SCRATCH_*scr, unsigned char bus,
		    Region&nmap, Region&pmap, Region&imap,
		    unsigned start, unsigned end);

      void map_irq_();

    private: // Not implemented
      BIOS32(const BIOS32&);
      BIOS32& operator= (const BIOS32&);
};


/*
 * $Log: BIOS32.h,v $
 * Revision 1.10  1997/11/03 23:25:40  steve
 *  Support PCI bus reset on command.
 *
 * Revision 1.9  1997/08/27 04:48:52  steve
 *  Move PCI IO to i960Rx.
 *
 * Revision 1.8  1997/08/26 20:05:48  steve
 *  Add support for IO regions. Map them and provide
 *  functions to access the targets.
 *
 * Revision 1.7  1997/08/17 01:17:32  steve
 *  Add a means of listing all the devices.
 *
 * Revision 1.6  1997/04/30 16:53:16  steve
 *  Add initial interrupt support to BIOS32,
 *  add some more ISO C library functions to ucr.
 *
 * Revision 1.5  1997/04/19 05:48:24  steve
 *  Redo the BIOS allocator to be simpler and to
 *  properly handle holes.
 *
 * Revision 1.4  1997/03/28 03:23:05  steve
 *  config space IO checks for errors.
 *
 * Revision 1.3  1997/03/27 00:08:53  steve
 *  Add the PLX9060 PCI side driver, and make
 *  the BIOS32 support short word config cycles.
 *
 * Revision 1.2  1997/02/14 21:42:30  steve
 *  PCI probe that properly handles prefetchable memory.
 *
 * Revision 1.1  1997/02/12 20:27:04  steve
 *  Support PCI BIOS behavior.
 *
 */

#endif
