#ifndef __chips_PLX9080p_H
#define __chips_PLX9080p_H
/*
 * Copyright (c) 1999 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: PLX9080p.h,v 1.10 1998/07/13 23:56:17 steve Exp $"

/*
 * This class supports the PLX PCI9060 PCI side. Use this class to
 * drive devices that use the PCI9080 interface chip from the PCI side.
 */

# include  <stddef.h>
# include  <assert.h>
class BIOS32;

class PLX9080p {

    private:
      volatile unsigned long&dword_(volatile void*base, unsigned off)
      { return * (volatile unsigned long*)((char*)base + off); }

      const volatile unsigned long&dword_(const volatile void*base,
					  unsigned off) const
      { return * (const volatile unsigned long*) ((const char*)base + off); }

      volatile unsigned char&byte_(volatile void*base, unsigned off)
      { return * ((volatile unsigned char*)base + off); }

      const volatile unsigned char&byte_(const volatile void*base,
					 unsigned off) const
      { return * ((const volatile unsigned char*)base + off); }

      volatile unsigned short&word_(volatile void*base, unsigned off)
      { return * ((volatile unsigned short*)base + off/2); }

      const volatile unsigned short&word_(const volatile void*base,
					  unsigned off) const
      { return * ((const volatile unsigned short*)base + off/2); }

    public:
	// The constructor takes the PCI bus and device/function
	// numbers for the desired part and locates all the registers
	// on its own.
      explicit PLX9080p (const BIOS32*pci, unsigned char bus,
			 unsigned char dev_fn);
      ~PLX9080p();

	// Write or read mailbox registers of the PLX. The PLX has 8
	// of these, numbered from 0 - 7.
      unsigned long mailbox(unsigned idx, unsigned long val)
      { return (dword_(plx_base_, 0x40+idx*4) = val); }

      unsigned long mailbox(unsigned idx) const
      { return dword_(plx_base_, 0x40+idx*4); }


	// There are two pins on the PLX chip that support random
	// things, including LEDS, etc. These methods write a value to
	// the single output pin, or read values from the input pin.
      bool user_out(bool bit)
      { if (bit) dword_(plx_base_, 0x6c) |= 0x10000;
        else dword_(plx_base_, 0x6c) &= ~0x10000;
        return bit;
      }

      bool user_out() const
      { return 0 != (dword_(plx_base_, 0x6c) & 0x10000); }

      bool user_in() const
      { return 0 != (dword_(plx_base_, 0x6c) & 0x20000); }


	// Use reset_peripheral(true) to force the peripherals on the
	// local side of the PLX into a reset state, and hold them
	// there. The reset will remain until later removed by calling
	// reset_peripheral(false).

      void reset_peripheral(bool flag)
      { if (flag) dword_(plx_base_, 0x6c) |= 0x40000000;
        else dword_(plx_base_, 0x6c) &= ~0x40000000;
      }


	// Set the PCI command codes to use for DMA and direct master
	// reads and writes.
      void set_pci_command_codes(unsigned short val)
       { word_(plx_base_, 0x6c) = val; }

	// The PCI9080 can do endian swaps. The stride of the swap (2
	// or 4 bytes) depends on the width of the local bus.
      void set_dma0_bigendian() { dword_(plx_base_, 0x0c) |= 0x80; }
      void set_dma1_bigendian() { dword_(plx_base_, 0x0c) |= 0x40; }

      void bus_enable(bool flag)
      { if (flag) dword_(plx_base_, 0x04) |=1;
        else dword_(plx_base_, 0x04) &= ~1;
      }

      void enable_irq(bool flag=true)
	    { if (flag) dword_(plx_base_, 0x68) |= 0x100;
	      else dword_(plx_base_, 0x68) &= ~0x100;
	    }

      void enable_bus_irq(bool flag=true)
	    { if (flag) dword_(plx_base_, 0x68) |= 0x900;
	      else dword_(plx_base_, 0x68) &= ~0x800;
	    }

	// These registers access the local-side bus of the PLX, and
	// thus the logic being interfaced to.

      volatile unsigned long&bus_dword(unsigned off)
      { return dword_(usr0_base_, off); }

      volatile unsigned short&bus_word(unsigned off)
      { return word_(usr0_base_, off); }

      volatile unsigned char&bus_byte(unsigned off)
      { return byte_(usr0_base_, off); }

      const volatile unsigned long&bus_dword(unsigned off) const
      { return dword_(usr0_base_, off); }

      const volatile unsigned short&bus_word(unsigned off) const
      { return word_(usr0_base_, off); }

      volatile unsigned long&bus1_dword(unsigned off)
      { return dword_(usr1_base_, off); }

      volatile unsigned short&bus1_word(unsigned off)
      { return word_(usr1_base_, off); }

    public:

	// properly stop the DMA channel, no matter what.
      void dma0_stop();
      void dma1_stop();

	// These are methods for controlling DMA behavior. There are
	// two DMA channels (0 and 1) that can transfer bytes
	// independently.
      void set_dma0_bus(unsigned long val)
      { dword_(plx_base_, 0x80) = val; }

      unsigned long get_dma0_bus() const
      { return dword_(plx_base_, 0x80); }

      void set_dma0_paddr(unsigned long addr)
      { dword_(plx_base_, 0x84) = addr; }

      void set_dma0_laddr(unsigned long laddr)
      { dword_(plx_base_, 0x88) = laddr; }

      void set_dma0_count(unsigned long cnt)
      { dword_(plx_base_, 0x8c) = cnt; }

      void set_dma0_descr(unsigned long addr, bool rd_wr,
			  bool intr, bool end_of_chain,
			  bool pci_local)
      { unsigned long val = addr;
        assert( (addr & 0xf) == 0 );
	if (rd_wr) val |= 8;
	if (intr)  val |= 4;
	if (end_of_chain) val |= 2;
	if (pci_local) val |= 1;
	dword_(plx_base_, 0x90) = val;
      }

      unsigned char dma0_cmd_status() const
      { return byte_(plx_base_, 0xa8); }

      void dma0_cmd_status(unsigned char val)
      { byte_(plx_base_, 0xa8) = val; }

      void set_dma1_bus(unsigned long val)
      { dword_(plx_base_, 0x94) = val; }

      unsigned long get_dma1_bus() const
      { return dword_(plx_base_, 0x94); }

      void set_dma1_paddr(unsigned long addr)
      { dword_(plx_base_, 0x98) = addr; }

      void set_dma1_laddr(unsigned long laddr)
      { dword_(plx_base_, 0x9c) = laddr; }

      void set_dma1_count(unsigned long cnt)
      { dword_(plx_base_, 0xa0) = cnt; }

      void set_dma1_descr(unsigned long addr, bool rd_wr,
			  bool intr, bool end_of_chain,
			  bool pci_local)
      { unsigned long val = addr;
        assert( (addr & 0xf) == 0 );
	if (rd_wr) val |= 8;
	if (intr)  val |= 4;
	if (end_of_chain) val |= 2;
	if (pci_local) val |= 1;
	dword_(plx_base_, 0xa4) = val;
      }

      unsigned char dma1_cmd_status() const
      { return byte_(plx_base_, 0xa9); }

      void dma1_cmd_status(unsigned char val)
      { byte_(plx_base_, 0xa9) = val; }

      void set_dma_thresholds(unsigned long val)
      { dword_(plx_base_, 0xb0) = val; }

	// Access the NVRAM that PLX chips use to store configuration
	// stuff. It is up to the caller to know that the memory is
	// the right size. The data sizes are in units of 16bit words,
	// and the addr is a word offset.

	// The PLX9080 supports 93CS46 and CS56 EEPROMS. The CS46 uses
	// a 6 bit address, the CS56 an 8 bit. The caller must get
	// that value right.
    public:
      bool nvram_present() const
      { return dword_(plx_base_, 0x6c) & 0x10000000; }

      void nvram_read(unsigned addr, unsigned short*data,
		      size_t ndata, unsigned naddr = 6);
      void nvram_write(unsigned addr, const unsigned short*data,
		       size_t ndata, unsigned naddr = 6);

    private:
      const BIOS32*const pci_;
      const unsigned char bus_, dev_fn_;

      volatile void*plx_base_;
      volatile void*usr0_base_;
      volatile void*usr1_base_;

      void nvram_send_bits_(unsigned long bits, unsigned nbits);

    private: // Not implemented
      PLX9080p(const PLX9080p&);
      PLX9080p& operator= (const PLX9080p&);
};

/*
 * $Log: PLX9080p.h,v $
 * Revision 1.10  1998/07/13 23:56:17  steve
 *  A few more access methods.
 *
 * Revision 1.9  1998/07/01 00:36:13  steve
 *  Method for setting PCI command codes.
 *
 * Revision 1.8  1998/06/16 00:31:55  steve
 *  Add stop for dma1
 *
 * Revision 1.7  1998/06/11 19:07:12  steve
 *  Add a DMA stop method.
 *
 * Revision 1.6  1998/06/06 04:39:45  steve
 *  Better abstraction of pci9080.
 *
 * Revision 1.5  1998/06/06 01:51:12  steve
 *  Support for endian swapping.
 *
 * Revision 1.4  1998/06/03 17:36:21  steve
 *  Add Access to local bus retion 1.
 *
 * Revision 1.3  1998/04/25 05:07:16  steve
 *  Support 93CS56 by adding address length parameter.
 *
 * Revision 1.2  1998/04/02 02:43:09  steve
 *  IRQ enable methods.
 *
 * Revision 1.1  1998/03/05 03:08:08  steve
 *  Add the PLX9080p class.
 *
 */
#endif
