#ifndef __gdb_GDB_H
#define __gdb_GDB_H
/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: GDB.h,v 1.7 1997/06/16 05:14:41 steve Exp $"
/*
 * $Log: GDB.h,v $
 * Revision 1.7  1997/06/16 05:14:41  steve
 *  New channel protocol design and implementation.
 *
 * Revision 1.6  1996/10/31 00:40:40  steve
 *  Add pause for explicit breakpoints.
 *
 * Revision 1.5  1996/10/28 03:45:14  steve
 *  License notice.
 *
 * Revision 1.4  1996/09/09 23:52:20  steve
 *  Get GDB to notice attentions and wakeup wait.
 *
 * Revision 1.3  1996/09/03 21:35:37  steve
 *  Redesign the BUS class to use a packet protocol instead of
 *  the channel oriented protocol.
 *
 *  Remove use of the Selector from GDB, becuase the new
 *  BUS interface doesn't support it.
 *
 * Revision 1.2  1996/08/30 23:38:33  steve
 *  Add the Selector class, and write into the Channel
 *  protocol some support for it.
 *
 * Revision 1.1  1996/08/28 00:40:30  steve
 *  Add the GDB proxy to uCR.
 *
 */

/*
 * This class implements a proxy for GDB within a target. It uses the
 * specified communications device to communicate with a properly
 * modified gdb on the user's computer.
 *
 * A gdb instance is created somewhere, then started with the go()
 * method in its own thread. Once started, the GDB object can controll
 * other threads and manipulate memory. It is important to put GDB in
 * its own thread, so that when it is executing, the thread being
 * inspected is certainly paused.
 *
 * NOTE: Currently, this GDB supports only BUS::Channels as
 * communications devices. In theory, this class could support
 * different kinds of communications devices, such as simple serial if
 * that were provided, but for now no.
 */

# include  <chips/BUS.h>
# include  <ucr/ThreadQueue.h>
# include  <stddef.h>

struct FAULT_RECORD;
struct gdb_if;

class GDB {

    public:
      explicit GDB(BUS::Channel*port);
      ~GDB();

    public:
      void go() __attribute__ ((noreturn));

	// This method works kinda like an explicit breakpoint,
	// allowing the thread to block pending intervention from the
	// debugger.
      void pause();

    private:

      THREAD*self_;

	// These private methods implement all the various debugger
	// commands. The go() method looks these up in a method table.
      void cmd_noop_(gdb_if&, const void*);
      void cmd_write_(gdb_if&, const void*);
      void cmd_read_(gdb_if&, const void*);
      void cmd_threads_(gdb_if&, const void*);
      void cmd_get_regs_(gdb_if&, const void*);
      void cmd_set_regs_(gdb_if&, const void*);
      void cmd_thread_test_(gdb_if&, const void*);
      void cmd_wait_(gdb_if&, const void*);
      void cmd_resume_(gdb_if&, const void*);
      void cmd_set_break_(gdb_if&, const void*);
      void cmd_clr_break_(gdb_if&, const void*);

	// Faults are delivered here. GDB uses these to collect the
	// threads that are faulty, and to synchronize them with the
	// host debugger.
      ThreadQueue faulty_;
      static bool handle_fault_(GDB*);

	// The wait_ request from the host resumes the threads, then
	// uses the wait_sync_ method to wait for something
	// interesting to happen.
      ISync wait_sync_;
      static bool wait_test_(GDB*);

      bool  wait_wakeup_flag_;
      static void attention_(GDB*obj, unsigned short, unsigned long);

      ThreadQueue ready_;

	// The trace_ list holds the threads stopped for trace
	// events. These are mentioned to GDB first, before any other
	// faults.
      ThreadQueue trace_;

	// Communicate with the host debugger with the BUS::Channel
	// object. The recv_ and reply_ simplify the I/O.
      BUS::Channel*const port_;

      char*recv_(gdb_if&);
      void reply_(const gdb_if&, const void* =0, size_t =0);

    private: // Not implemented
      GDB(const GDB&);
      GDB& operator= (const GDB&);
};

#endif
