#ifndef __mc68322_PrintEngine_H
#define __mc68322_PrintEngine_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: PrintEngine.h,v 1.6 1996/10/28 03:45:19 steve Exp $"
/*
 * $Log: PrintEngine.h,v $
 * Revision 1.6  1996/10/28 03:45:19  steve
 *  License notice.
 *
 * Revision 1.5  1995/12/14 22:19:08  steve
 *  Provide page synchronization.
 *
 * Revision 1.4  1995/11/07 07:54:48  steve
 *  Support optional smart-toner mode.
 *
 * Revision 1.3  1995/10/25 04:04:48  steve
 *  Support Automatic printer resolution switching.
 *
 * Revision 1.2  1995/10/09 01:28:57  steve
 *  Keep slightly better track of errors.
 *
 * Revision 1.1  1995/09/02 18:27:08  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

/*
 * This class represents a print engine connected to a m68322. For
 * details see the MC68322 Integrated Printer Processor User's Manual.
 *
 * A PrintEngine behaves like a FIFO queue of bands--put bands in one
 * end and pull them out again from the other end. Once placed in the
 * queue, the printer holds onto the band until it is printed. It is
 * only safe to alter or destroy a band after it is pulled from the
 * other end. The printer does not modify any of the fields in the
 * band, except the next_ member when it is putting it in queues.
 *
 * GEOMETRY:
 * The PrintEngine vertical resolution is normally fixed by the design
 * of the engine, but horizontal resolution can be to some degree
 * controlled, as can margins. The application supplies certain
 * control bits for each geometry specification, then selects in the
 * band the desired page geometry by number.
 *
 * The default geometry is 0, so at least one is required.
 *
 * CTRL:
 * The PrintEngine unly uses the PLE, PLD and SDN fields from the ctrl
 * register to fill in the real ctrl register of the engine. See the
 * MC68322 Procerror User's Manual, section 10.1.2 for details.
 *
 * The ctrl field controlls the DPI resolution (horizontal) by
 * specifying certain values to be loaded into the PVC Control
 * Register. The PLS and PLD registers are used for this purpose.
 *
 * The ctrl field cal also set the toner saver values for the
 * page. These bits only apply if the band has the toner_saver flag
 * set.
 *
 * The left_margin varies depending on the horizontal resolution (and
 * possibly the application) so is collected from the Geometry. The
 * top_margin is generally fixed. The application may choose to place
 * the image on the page by adjusting this.
 *
 * NOTE: Although this class deals with interrupts (for itself) just
 * fine, it is not thread-safe.
 */

# include  <ucr/ISync.h>
# include  <mc68322/Band.h>

class PrintEngine {

    public:

      struct Geometry {
	    unsigned short top_margin;
	    unsigned short left_margin;
	    unsigned short ctrl;

	      // OR these values into the ctrl, if desired.
	    enum { HT_HALF    = 0x0000,
		   HT_QUARTER = 0x4000,
		   HT_SIXTH   = 0x8000,
		   HT_EIGTH   = 0xc000 };
      };


        // Construct a PrintEngine with the address of the beginning
	// of the mc68322 memory mapped registers. The engine can also
	// place its interrupts at any interrupt level, so specify the
	// desired value here.
      PrintEngine(const Geometry*speeds, unsigned char nspeeds,
		  unsigned char level, unsigned long base = 0xfff000);


        // Send a band to be printed whenever the printer is
	// ready. Pulling a band will block until there is a band
	// ready to be pulled.

      void  print_band(Band*);
      Band* pull_completed_band();

        // Start the printer feeding a page to be printed. This method
	// will block until the Page Sync is sent from the
	// printer. The print_band() method will start the page
	// implicitly, but this can be used to carefully synchronize
	// with the engine.

      void  page_sync();


        // Send a command to the printer, and return the result
	// code. Every command has a single status, and a status comes
	// only in response to a command. (Some commands are status
	// requests.) If the wait_flag is false, then do not wait for
	// the status, get it later with get_status().

        // get_status() will block waiting for any outstanding command
	// to complete.
      unsigned char do_command(unsigned char cmd, bool wait_flag = true);
      unsigned char get_status() const;


    private:

        // These private types describe registers in the 68322 that
	// control printer operations. Their addresses are guessed
	// from the mc68322 base address passed to the constructor.

      struct PVC_ {
	    volatile unsigned short print;
	    volatile unsigned short pvc_flags;

	    volatile unsigned short vmargin, hmargin;
	    volatile unsigned short height, width;
	    volatile unsigned short pcb_flags;

	    unsigned short pad;

	    volatile unsigned long image_bit_address;
      };

      struct PCOM_ {
	    volatile unsigned short command;
	    volatile unsigned short status;
	    unsigned short pcom_flags;
      };

      struct INTR_ {
	    volatile unsigned short event_enable;
	    volatile unsigned short event_flags;
	    volatile unsigned short event_level;

	    unsigned short pad[13];

	    volatile unsigned short com_enable;
	    volatile unsigned short com_flags;
	    volatile unsigned short com_level;
      };

      PVC_ *const pvc_;
      PCOM_*const pcom_;
      INTR_*const intr_;

      const Geometry*const speeds_;
      const unsigned char nspeeds_;
      const unsigned short ctrl_mask_;

        // Manage the state of delivering a command and receiving back
	// the status.
    private:
      unsigned char com_status_;
      mutable volatile bool com_status_ready_;
      mutable ISync com_sync_;
      void intr_com_(unsigned short);

        // Manage the state of delivering video data to the printer. A
	// band starts in the pending_queue_, moves to the
	// printing_band_, then finally to the done_queue_ where the
	// application can retrieve it.
    private:
      ISync     print_sync_;
      BandQueue pending_queue_;
      BandQueue printing_queue_;
      BandQueue done_queue_;


      bool      top_of_page_;
      bool  psync_flag_;
      ISync psync_sync_;
      void load_bands();

      unsigned  printing_begun_;
      void intr_video_(unsigned short);

    private:
      static void intr(PrintEngine*obj, unsigned vec);

    private: // not implemented
      PrintEngine(const PrintEngine&);
      PrintEngine& operator= (const PrintEngine&);
};

#endif
