#ifndef __ucr_ISync_H
#define __ucr_ISync_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ISync.h,v 1.6 1998/01/01 20:45:15 steve Exp $"
/*
 * $Log: ISync.h,v $
 * Revision 1.6  1998/01/01 20:45:15  steve
 *  ISync has const methods.
 *
 * Revision 1.5  1996/10/28 03:45:22  steve
 *  License notice.
 *
 * Revision 1.4  1995/10/15 21:54:05  steve
 *  The sleep that understands bit masks now
 *  handles on and off tests, with don't care bits.
 *
 * Revision 1.3  1995/10/15 05:27:31  steve
 *  Add a sleep that understands bit masks, and
 *  split up the ISync.cc source file.
 *
 * Revision 1.2  1995/09/11 01:13:55  steve
 *  Get rid of warnings about volatile.
 *
 * Revision 1.1  1995/09/02 18:27:10  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

/*
 * This class is meant as a means for ISRs to synchronize with
 * threads. The wakeup() method may switch to threads, but will never
 * block. The methods are also protected from ISR interruption as
 * needed.
 *
 * This class behaves much like the good old UNIX sleep/wakeup
 * primitives that Unix device driver writers have come to know. A
 * thread calls sleep to wait for something to change, and an
 * interrupt handler calls wakeup to awaken all the threads sleeping
 * on the condition.
 *
 * The difference is, the test function is passed to the sleep method,
 * and sleep does the checking. This saves the caller the effort of
 * blocking interrupts around the test.
 *
 * If you call sleep() with a pointer to a bool, then that variable
 * will be tested instead of a function. This may be more convenient
 * when the isr does smart things and sets a flag when done.
 *
 * If you call sleep() with a reference to a volatile unsigned short and
 * a mask, the bits in the short will be tested against the mask, and
 * if all the bits in the mask are set in the word the sleep will be
 * eligible for awakening.
 *
 * The thread code calls the sleep() method, and the interrupt handler
 * calls the wakeup() method. Note that calls to wakeup() may cause
 * the ISR to switch out for another thread, which may have interrupts
 * enabled, so an ISR may reenter here with the context of a different
 * thread.
 */

# include  <ucr/ThreadQueue.h>



class ISync {

    public:
      typedef bool (*funp)(const volatile void*);
      ISync();
      void sleep(funp fun, const volatile void*) const;

      void sleep(const volatile bool*) const;
      void sleep(const volatile unsigned short&word,
		 unsigned short on_mask,
		 unsigned short off_mask = 0);

      void wakeup();

    private:
      mutable ThreadQueue stuck_;

    private: // Not implemented
      ISync(const ISync&);
      ISync& operator= (const ISync&);
};

#endif
