#ifndef __ucr_ThreadQueue_H
#define __ucr_ThreadQueue_H

/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ThreadQueue.h,v 1.6 1997/08/11 22:39:14 steve Exp $"
/*
 * $Log: ThreadQueue.h,v $
 * Revision 1.6  1997/08/11 22:39:14  steve
 *  Threads that finish are dequeued.
 *
 * Revision 1.5  1996/10/28 03:45:25  steve
 *  License notice.
 *
 * Revision 1.4  1996/09/11 20:20:14  steve
 *  Minor optimizations
 *
 * Revision 1.3  1996/08/08 20:57:21  steve
 *  Minor optimizations by inlining a few things
 *  and setting optimization attributes.
 *
 * Revision 1.2  1996/08/08 05:22:38  steve
 *  Get rid of the __Scheduler class, it was not
 *  needed. Add the thread_runnable and _reschedule
 *  functions for thread scheduling.
 *
 * Revision 1.1  1995/09/02 18:27:11  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */

struct THREAD;

/*
 * ThreadQueue objects can hold threads. Threads can only be in one
 * thread queue, and cannot be in the run queue while in a thread
 * queue. If a thread is put into a thread queue, it will
 * automatically be removed from any other thread queue (including the
 * run queue).
 *
 * The above property makes this class perfect for implementing
 * synchronization and scheduling primitives. Remember that threads
 * will only exist in one queue at a time, always.
 *
 * NOTE: No special precautions have been taken in ThreadQueue to
 * protect the implementation from interrupts. That is up to the
 * application, for now.
 *
 * NOTE: Although putting a thread into a queue may take it out of the
 * run queue, it will not cause a thread switch. Use thread_reschedule() to
 * cause a reschedule, if necessary.
 *
 * NOTE: thread_switch, or any function that calls thread_switch, will
 * pull the thread out of any ThreadQueue it might be in.
 */

class ThreadQueue {

    public:
      ThreadQueue() : last_(0) { }
      ~ThreadQueue() { }

	// This constructor will create a thread with exactly one
	// thing in it. The thread will be pulled from any other queue
	// before this finished.
      explicit ThreadQueue(THREAD*);

        // Enqueue will put the thread on the end of the queue, or the
	// front if the flag is set. It will pull it from another
	// queue if necessary.
      void enqueue(THREAD*, bool front = false);

        // Pull removes the thread from the queue. Remove either the
	// explicit thread, or the first thread in the queue. Peek
	// will return the first thread without pulling it.
      THREAD* pull();
      THREAD* peek();

	// This method removed the thread from any ThreadQueue that it
	// might be in. It is needed for example when the thread is
	// ready to be destroyed.
      static void dequeue(THREAD*);

    private:
      THREAD* last_;
      void pull_(THREAD*);

    private:
      ThreadQueue(const ThreadQueue&);
      ThreadQueue& operator= (const ThreadQueue&);
};

#endif
