/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BIOS32.cc,v 1.19 1998/07/01 00:35:27 steve Exp $"

/*
 * XXXX
 * The probing currently assumes a few things:
 * - bridges have no valid BARs
 * - There are no I/O regions
 * - prefetchable regions of bridges not supported.
 * - this is an i960RP and I'm using direct mapping.
 * - each device has exactly one function unit.
 * - cache line size is 16 dwords
 */

# include  <chips/BIOS32.h>
# include  <ucr/param.h>
# include  "region.h"
# include  <assert.h>


/*
 * For now, assume we are talking about the i960RP secondary
 * PCI. Also, assume for now that the host BIOS has had its chance.
 */

/* from the PCI-to-PCI bridge unit */
static volatile unsigned char& SBNR   = *(unsigned char*)0x1019;
static volatile unsigned short&MBR    = *(unsigned short*)0x1020;
static volatile unsigned short&MLR    = *(unsigned short*)0x1024;
static volatile unsigned short&SMBR   = *(unsigned short*)0x1058;
static volatile unsigned short&SMLR   = *(unsigned short*)0x105a;
static volatile unsigned short&SDER   = *(unsigned short*)0x105c;

/* these are in the ATU */
static volatile unsigned long& SOIOWVR= *(unsigned long*)0x126c;
static volatile unsigned long& ATUCR  = *(unsigned long*)0x1288;
static volatile unsigned long& SATUISR= *(unsigned long*)0x1294;
static volatile unsigned long& SATUCMD= *(unsigned long*)0x1298;
static volatile unsigned long& SOCCAR = *(unsigned long*)0x12a8;
static volatile unsigned long& SOCCDR = *(unsigned long*)0x12b0;

static bool type0_read_dword(unsigned char dev_fn,
			     unsigned char reg_id,
			     unsigned long&result)
{
      unsigned long addr = 0x10000 << (dev_fn >> 3);
      addr |= reg_id;

      SOCCAR = addr;
      SATUISR = 0x08;

      result = SOCCDR;
      return ! (SATUISR & 0x08);
}

static bool type0_write_dword(unsigned char dev_fn,
			      unsigned char reg_id, unsigned long value)
{
      unsigned long addr = 0x10000 << (dev_fn >> 3);
      addr |= reg_id;

      SOCCAR = addr;
      SATUISR = 0x08;

      SOCCDR = value;
      return ! (SATUISR & 0x08);
}

static bool type1_read_dword(unsigned char bus,
			     unsigned char dev_fn,
			     unsigned char reg_id,
			     unsigned long&result)
{
      unsigned long addr = (bus << 16) + (dev_fn << 8) + reg_id + 1;
      SOCCAR = addr;
      SATUISR = 0x08;

	// There is no hardware way to tell that there is no device
	// past the bridge, so use the fact that missing registers
	// must read as 0xffffffff to detect errors on the other side
	// of the bridge.
      result = SOCCDR;
      return result != 0xffffffff;
}

static bool type1_write_dword(unsigned char bus,
			      unsigned char dev_fn,
			      unsigned char reg_id, unsigned long value)
{
      unsigned long addr = (bus << 16) + (dev_fn << 8) + reg_id + 1;
      SOCCAR = addr;
      SATUISR = 0x08;

      SOCCDR = value;
      return ! (SATUISR & 0x08);
}

BIOS32::BIOS32(unsigned long mask)
{
      SCRATCH_ scr[DEV_LIMIT_];

      ATUCR |= 4;
      SATUCMD |= 4;

	// Mark the low 2-gig as a private memory space. This will
	// prevent the PCI-to-PCI bridge forwarding things that are
	// here over to the primary side. The ``4'' in the SDER tells
	// the bridge that the limit registers mark a region to NOT be
	// forwarded to the secondary side.
      SMBR = 0;
      SMLR = 0x7fff;
      SDER = 4;

      my_bus = SBNR;

      ndevs_ = 0;
      map_vga_ = false;
      probe_bus_(scr, my_bus, mask);

	// Use the direct map area of the i960 for mapping accessible
	// devices. Take out of the map area a hole for reserved stuff
	// in the beginning.
      Region map (0, 0x80000000);
      map.subtract(0, 0xffffe000);

	// Use a translation window for mapping a fixed region to the
	// place where I put the I/O devices. Put this at low
	// addresses and pull values from there.
      Region imap(0, 0x1000);
      SOIOWVR = 0;

	// Subtract out the VGA regions, which are, thanks to certain
	// PC and O/S manufacturers, fixed. Blech! Anyhow, this only
	// applies if there is a VGA device installed.
      if (map_vga_) {
	    map.subtract(0xa0000, 0xfffe0000);
	    imap.subtract(0x03c0, 0xffe0);
	    imap.subtract(0x03b0, 0xfff0);
      }

	// For now, assume that the host doesn't use any addresses
	// below the first two gig, which is the direct map area.

      assert( (unsigned long)(MBR << 16) > 0x80000000 );
      assert( (unsigned long)(MLR << 16) > 0x80000000 );

	// map all the devices I located. Starting in the top here, I
	// treat the prefetchable and non-prefetchable memory the same.
      map_bus_(scr, my_bus, map, map, imap, 0, ndevs_);
      map_irq_();

	// Write to the ATUCR to turn on direct addressing and send
	// direct addresses to the secondary PCI bus.
      ATUCR |= 0x180;
}

/*
 * This method runs through the bus, locating devices and configuring
 * the bus numbers in bridges. The SCRATCH contains the distribution
 * of allocation requests, and a later pass will actually arrange the
 * BARs in memory. Keep the prefetchable memory separate from the
 * non-prefetchable memory so that the mapper can put them into
 * seperate bridge regions.
 *
 * The support for a mask allows the caller to prevent certain devices
 * being probed. This supports for example devices configured by other
 * means. The mask is not passed to recursions.
 */
void BIOS32::probe_bus_(SCRATCH_*scr, unsigned char bus,
			unsigned long mask, SCRATCH_*bridge)
{

      for (unsigned dev = 0 ;  dev < 32 ;  dev += 1) {
	      // Skip devices that are masked out
	    if (mask & (1 << dev)) continue;

	      // Probe the device. If it is not there, the configure
	      // read will fail and I just skip it.
	    unsigned long id;
	    if (! read_config_dword(bus, dev << 3, 0, id)) continue;

	      // Ah, got a valid id so on we go.
	    unsigned cur = ndevs_++;
	    devs_[cur].bus = bus;
	    devs_[cur].dev_fn = dev << 3;
	    devs_[cur].id = id;

	    unsigned long type;
	    read_config_dword(bus, devs_[cur].dev_fn, 8, type);
	    if ((type&~0xff) == 0x06040000) {

		  devs_[cur].is_bridge = true;

		    // set bus number on the bridge by writing the
		    // current bus as the primary number, and bus+1 as
		    // the secondary bus. For probing, temporarily set a
		    // bus extent that is infinitely high. I will
		    // correct it with the discovered extent later.

		  unsigned char nbus = bus+1;

		  unsigned long tmp;
		  read_config_dword(bus, devs_[cur].dev_fn, 0x18, tmp);
		  tmp = 0xffff0000UL | (nbus<<8) | bus;
		  write_config_dword(bus, devs_[cur].dev_fn, 0x18, tmp);



		    // Probe for devices behind the bridge. This uses
		    // recursion to treat the next bus the same as
		    // this one. Round up to minimum region granularity.
		  scr[cur].nunder = scr[cur].punder = scr[cur].iunder = 0;
		  probe_bus_(scr, bus+1, 0, scr+cur);
		  scr[cur].nunder = (scr[cur].nunder+0xfffff) & ~0xfffff;
		  scr[cur].punder = (scr[cur].punder+0xfffff) & ~0xfffff;
		  scr[cur].iunder = (scr[cur].iunder+0x00fff) & ~0x00fff;

		    // Calculate the correct bus extent for the
		    // bridge, and write the final bus configuration
		    // into the bridge.
		  unsigned xbus = nbus;
		  for (unsigned idx = cur+1 ;  idx < ndevs_ ;  idx += 1)
			if (devs_[idx].bus > xbus) xbus = devs_[idx].bus;

		  tmp = 0xff000000UL | (xbus<<16) | (nbus<<8) | bus;
		  write_config_dword(bus, devs_[cur].dev_fn, 0x18, tmp);

		    // Do not allocate memory for the bridge.

		    // XXXX This is incorrect behavior, the bridge has
		    // 2 bars it can use. However, the bridges I see
		    // do not use them.
		  for (unsigned bar = 0 ;  bar < 6 ;  bar += 1) {
			scr[cur].nmem[bar] = 0;
			scr[cur].pmem[bar] = 0;
			scr[cur].imem[bar] = 0;
		  }

		    // Don't forget to add up this region.
		  if (bridge) {
			bridge->nunder += scr[cur].nunder;
			bridge->punder += scr[cur].punder;
			bridge->iunder += scr[cur].iunder;
		  }

	    } else {

		  devs_[cur].is_bridge = false;
		  if ((type&0xffffff00) == 0x03000000) map_vga_ = true;


		    // This is an ordinary device. Run through all the
		    // BARs looking for memory that it takes up.

		  scr[cur].nunder = 0;
		  scr[cur].punder = 0;
		  scr[cur].iunder = 0;
		  unsigned long nsum = 0, psum = 0, isum = 0;

		  for (unsigned bar = 0 ;  bar < 6 ;  bar += 1) {
			unsigned long tmp;
			unsigned reg = 0x10 + 4*bar;
			write_config_dword(bus, devs_[cur].dev_fn,
					   reg, 0xfffffff0);
			read_config_dword(bus, devs_[cur].dev_fn, reg, tmp);

			if ((tmp&0x0f) == 0x08) {
				// prefetchable memory
			      tmp = ~(tmp & ~15) + 1;
			      psum += tmp;
			      scr[cur].nmem[bar] = 0;
			      scr[cur].pmem[bar] = tmp;
			      scr[cur].imem[bar] = 0;
			} else if ((tmp&0x0f) == 0x00) {
				// non-prefetchable memory
			      tmp = ~(tmp & ~15) + 1;
			      nsum += tmp;
			      scr[cur].nmem[bar] = tmp;
			      scr[cur].pmem[bar] = 0;
			      scr[cur].imem[bar] = 0;
			} else if ((tmp&0x03) == 0x01) {
				// IO space
			      tmp = ~(tmp & ~0x03) + 1;
			      isum += tmp;
			      scr[cur].nmem[bar] = 0;
			      scr[cur].pmem[bar] = 0;
			      scr[cur].imem[bar] = tmp;
			} else {
			      scr[cur].nmem[bar] = 0;
			      scr[cur].pmem[bar] = 0;
			      scr[cur].imem[bar] = 0;
			}
		  }

		  if (bridge) {
			bridge->nunder += nsum;
			bridge->punder += psum;
			bridge->iunder += isum;
		  }
	    }

	      // This sets the cache line size to 16 dwords, and the
	      // latency timer to 32.
	      // XXXX I should run through a set of candidate cache
	      // XXXX line sizes and see what the devices can accept,
	      // XXXX and choose based on the largets that all can
	      // XXXX accept.
	    unsigned long tmp;
	    read_config_dword(bus, devs_[cur].dev_fn, 0x0c, tmp);
	    tmp = (tmp&0xffff0000) | 0x2010;
	    write_config_dword(bus, devs_[cur].dev_fn, 0x0c, tmp);

      }

}

static unsigned long size2mask(unsigned long size)
{
      if (size == 0) return 0xffffffff;

      size -= 1;
      unsigned long mask = 0xffffffff;
      while (mask & size) mask <<= 1;
      return mask;
}

/*
 * Now I located all the devices connected to me, run through them
 * and try to place them in address space. Set up base registers to
 * make the assignments, and set up limit registers for bridges.
 */
void BIOS32::map_bus_(SCRATCH_*scr, unsigned char bus,
		     Region&nmap, Region&pmap, Region&imap,
		     unsigned start, unsigned end)
{
      unsigned dev = start;

      while (dev != end) {

	    if (devs_[dev].is_bridge) { // THIS IS A BRIDGE

		  unsigned long nmask = size2mask(scr[dev].nunder);
		  unsigned long pmask = size2mask(scr[dev].punder);
		  unsigned long imask = size2mask(scr[dev].iunder);

		  unsigned long nregion = 0;
		  unsigned long pregion = 0;
		  unsigned long iregion = 0;

		  unsigned long b_enable = 0x0004; // bus mastering enabled...

		    // Calculate the limit register dword values for
		    // all the region types.

		  unsigned long nlim = 1, plim = 1, ilim = 1;
		  if (nmask != 0xffffffff) {
			b_enable |= 2;
			nregion = nmap.allocate(nmask);
			nlim = (nregion>>16) + (nregion+~nmask) + 1;
		  }

		  if (pmask != 0xffffffff) {
			b_enable |= 2;
			pregion = pmap.allocate(pmask);
			plim = (pregion>>16) + (pregion+~pmask) + 1;
		  }

		  if (imask != 0xffffffff) {
			b_enable |= 1;
			iregion = imap.allocate(imask);
			ilim = (iregion>>8) + (iregion+~imask) + 1;
		  }

		    // Locate the region of devices behind this bridge.
		  unsigned br = dev;
		  dev += 1;
		  for (;;) {
			if (dev == end) break;
			if (devs_[dev].bus == bus) break;
			dev += 1;
		  }

		  if (dev > (br+1)) {
			  // If there are any devices beyond the
			  // bridge, this condition maps them into the
			  // non-prefetchable and prefetchable
			  // regions.

			Region nmap2(nregion, nmask),
			       pmap2(pregion, pmask),
			       imap2(iregion, imask);

			map_bus_(scr, devs_[br+1].bus, nmap2, pmap2,
				 imap2, br+1, dev);


			  // Set up the limit registers for the memory
			  // regions.
			write_config_dword(devs_[br].bus, devs_[br].dev_fn,
					   0x20, nlim);

			write_config_dword(devs_[br].bus, devs_[br].dev_fn,
					   0x24, plim);

			write_config_dword(devs_[br].bus, devs_[br].dev_fn,
					   0x1c, ilim);

			  // Clear the upper 32 bits of the DAC address
			  // registers.
			write_config_dword(devs_[br].bus,
					   devs_[br].dev_fn, 0x28, 0);

			write_config_dword(devs_[br].bus,
					   devs_[br].dev_fn, 0x2c, 0);

			write_config_dword(devs_[br].bus,
					   devs_[br].dev_fn, 0x30, 0);


			  // Enable bus mastering and memory writes.

			write_config_dword(devs_[br].bus,
					   devs_[br].dev_fn, 0x04,
					   b_enable);

		  } else {
			  // There are no devices behind this bridge,
			  // so turn it completely off.

			write_config_dword(devs_[br].bus,
					   devs_[br].dev_fn, 0x04,
					   0x0000);
		  }

	    } else { // THIS IS NOT A BRIDGE

		    // This is a device (not a bridge) so go through
		    // all the BARs looking for memory regions to
		    // allocate. At this point I do not care if the
		    // memory is prefetchable.

		  for (unsigned bar = 0 ;  bar < 6 ;  bar += 1) {

			unsigned long base = 0;
			if (scr[dev].nmem[bar]) {
			      unsigned long mem = scr[dev].nmem[bar];
			      unsigned bar_mask = size2mask(mem);
			      base = nmap.allocate(bar_mask);

			} else if (scr[dev].pmem[bar]) {
			      unsigned long mem = scr[dev].pmem[bar];
			      unsigned bar_mask = size2mask(mem);
			      base = pmap.allocate(bar_mask);

			} else if (scr[dev].imem[bar]) {
			      unsigned long mem = scr[dev].imem[bar];
			      unsigned bar_mask = size2mask(mem);
			      base = nmap.allocate(bar_mask);
			}

			unsigned reg = 0x10 + bar*4;
			write_config_dword(devs_[dev].bus,
					   devs_[dev].dev_fn, reg,
					   base);


		  }

		    // Enable the device so it responds to memory
		    // accesses. Also, clear all the status bits and
		    // start fresh.
		  write_config_dword(devs_[dev].bus,
				     devs_[dev].dev_fn, 0x04, 0xff07);

		  dev += 1;
	    }
      }
}

/*
 * This guesses IRQ numbers from the device numbers of the PCI
 * device. The IRQ number chosen is the number that the device driver
 * can pass to the ucr library to gain access to the interrupts
 * generated by the device.
 *
 * XXXX This is extremely ISE board specific.
 */
void BIOS32::map_irq_()
{
      for (unsigned idx = 0 ;  idx < ndevs_ ;  idx += 1) {
	    unsigned short bus_dev_fn = ((devs_[idx].bus - my_bus) << 8)
		  + (devs_[idx].dev_fn & 0xf8);
	    unsigned long tmp;
	    read_config_dword(devs_[idx].bus, devs_[idx].dev_fn, 0x3c, tmp);

	    unsigned char irq = 0xff;

	      // if the device drives INTA#, assign it an irq based on
	      // its device and bus.
	    if ((tmp & 0xff00) == 0x100) switch (bus_dev_fn) {
		case 0x0100: // IMC
		  irq = XINT0;
		  break;
		case 0x0000: // Daughter card #1 (scanner)
		case 0x0008: // Daughter card #2
		  irq = XINT1;
		  break;
		case 0x0010: // Daughter card #3
		case 0x0020: // Daughter card #4 (scanner)
		  irq = XINT2;
		  break;
		case 0x0018: // (DEC BRIDGE generates no interrupts)
		  break;
		case 0x0128: // Daughter card #5
		  irq = XINT3;
		  break;
		case 0x0130: // Daughter card #6
		case 0x0138: // Daughter card #7
		  irq = XINT4;
		  break;
		case 0x0120: // CIP
		  irq = XINT5;
		  break;
	    }

	    tmp = (tmp & 0xffffff00) | irq;
	    write_config_dword(devs_[idx].bus, devs_[idx].dev_fn, 0x3c, tmp);
      }
}

bool BIOS32::find_device(unsigned short vendor_id,
		      unsigned short device_id,
		      unsigned inst,
		      unsigned char&bus, unsigned char&dev_fn) const
{
      unsigned count = 0;
      unsigned long id = (device_id << 16) | vendor_id;

      for (unsigned idx = 0 ; (idx < DEV_LIMIT_) && devs_[idx].id ; idx += 1)

	    if (devs_[idx].id == id)
		  if (count == inst) {
			bus = devs_[idx].bus;
			dev_fn = devs_[idx].dev_fn;
			return true;
		  } else {
			count += 1;
		  }


      return false;
}

bool BIOS32::read_config_dword(unsigned char bus, unsigned char dev_fn,
			       unsigned char reg_id, unsigned long&out)
      const
{
      if (bus == my_bus) return type0_read_dword(dev_fn, reg_id, out);
      else return type1_read_dword(bus, dev_fn, reg_id, out);
}

bool BIOS32::write_config_dword(unsigned char bus, unsigned char dev_fn,
				unsigned char reg_id, unsigned long value)
      const
{
      if (bus == my_bus) return type0_write_dword(dev_fn, reg_id, value);
      else return type1_write_dword(bus, dev_fn, reg_id, value);
}
/*
 * $Log: BIOS32.cc,v $
 * Revision 1.19  1998/07/01 00:35:27  steve
 *  Set latency timers.
 *
 * Revision 1.18  1998/01/29 22:01:20  steve
 *  Configure cache line size.
 *
 * Revision 1.17  1997/11/03 23:25:41  steve
 *  Support PCI bus reset on command.
 *
 * Revision 1.16  1997/08/27 04:48:52  steve
 *  Move PCI IO to i960Rx.
 *
 * Revision 1.15  1997/08/26 20:05:49  steve
 *  Add support for IO regions. Map them and provide
 *  functions to access the targets.
 *
 * Revision 1.14  1997/08/02 04:32:00  steve
 *  Correct IRQ assignment for ISE board rev 3.
 *
 * Revision 1.13  1997/07/18 23:44:36  steve
 *  Handle case of no devices behind bridge.
 *
 * Revision 1.12  1997/06/04 03:33:57  steve
 *  Tell the bridge local devices are private.
 *
 * Revision 1.11  1997/05/02 16:56:51  steve
 *  Fix problem converting size 2 mask.
 *
 * Revision 1.10  1997/04/30 16:53:17  steve
 *  Add initial interrupt support to BIOS32,
 *  add some more ISO C library functions to ucr.
 *
 * Revision 1.9  1997/04/19 05:48:25  steve
 *  Redo the BIOS allocator to be simpler and to
 *  properly handle holes.
 *
 * Revision 1.8  1997/04/17 04:01:56  steve
 *  spelling error
 *
 * Revision 1.7  1997/03/28 03:23:09  steve
 *  config space IO checks for errors.
 *
 * Revision 1.6  1997/03/27 00:08:54  steve
 *  Add the PLX9060 PCI side driver, and make
 *  the BIOS32 support short word config cycles.
 *
 * Revision 1.5  1997/03/24 05:55:27  steve
 *  more thorough config of bridge.
 *
 * Revision 1.4  1997/03/14 18:01:26  steve
 *  Cleaner register references.
 *
 * Revision 1.3  1997/02/14 21:42:31  steve
 *  PCI probe that properly handles prefetchable memory.
 *
 * Revision 1.2  1997/02/14 04:04:25  steve
 *  Configure Bridges and devices more completely.
 *
 * Revision 1.1  1997/02/12 20:27:05  steve
 *  Support PCI BIOS behavior.
 *
 */
