/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: BUS.cc,v 1.22 1998/05/26 16:16:32 steve Exp $"


# include  <chips/BUS.h>
# include  <ucr/ucr.h>
# include  <assert.h>

# define ROOT_TABLE_MAGIC 0x1ead1eaf

# define ROOT_CBASE(x, id) ((x) +  136 + 8*(id))
# define ROOT_CMAGIC(x, id) ((x) +  140 + 8*(id))


BUS::BUS()
: channels_(0), dev_(0), root_table_ptr_(0)
{
}

BUS::~BUS()
{
}


/*
 * Connect a Channel object to a specific channel. Note that any
 * channel can be opened only once.
 */
void BUS::open_channel(BUS::Channel*cp)
{
      assert( cp->master_ == 0 );
      spl_cookie_t cookie = thread_splhi();
      cp->list_ = channels_;
      channels_ = cp;
      thread_spl(cookie);
      cp->master_ = this;
}

void BUS::close_channel(BUS::Channel*cp)
{
      spl_cookie_t cookie = thread_splhi();
      if (channels_ == cp) {
	    channels_ = cp->list_;

      } else {
	    BUS::Channel*prev = channels_;
	    while (prev->list_ != cp) {
		  assert( prev->list_ );
		  prev = prev->list_;
	    }
	    prev->list_ = cp->list_;
      }
      thread_spl(cookie);

      cp->master_ = 0;
}


/*
 * Read bits of the channel table for this channel, and check for
 * interesting events that the channel should know about.
 *
 * It is safe to call this in interrupt handlers, indeed that is the
 * best place.
 */
void BUS::check_channel_status_(const Channel*ch)
{
	/* Get the channel table from the root table. */
      ch->table_ = dev_->readl(ROOT_CBASE(root_table_ptr_,ch->channel_id()));
      ch->magic_ = dev_->readl(ROOT_CMAGIC(root_table_ptr_,ch->channel_id()));

	/* Check magic numbers for sanity. */
      if (ch->magic_ == 0)
	    return;

      if (dev_->readl(ch->table_+0) != ch->magic_) {
	    ch->magic_ = 0;
	    ch->table_ = 0;
	    return;
      }

      if (dev_->readl(ch->table_+4) != ch->table_) {
	    ch->magic_ = 0;
	    ch->table_ = 0;
	    return;
      }

      do {
	    unsigned long first_out = dev_->readl(ch->table_+16);
	    unsigned long next_out  = dev_->readl(ch->table_+20);

	    if (first_out == next_out)
		  break;

	    unsigned long addr = dev_->readl(ch->table_+32 + 8*first_out + 0);
	    if (ch->raddr_ == addr)
		  break;

	    unsigned long size = dev_->readl(ch->table_+32 + 8*first_out + 4);

	    ch->raddr_ = addr;
	    ch->rsize_ = size;
	    ch->roff_  = 0;
      } while(0);

      do {
	    unsigned long first_in = dev_->readl(ch->table_+24);
	    unsigned long next_in  = dev_->readl(ch->table_+28);

	    unsigned long step = (next_in + 1) % 4;

	    if (step == first_in)
		  break;

	    unsigned long addr = dev_->readl(ch->table_+64 + 8*next_in + 0);
	    if (ch->waddr_ == addr)
		  break;

	    unsigned long size = dev_->readl(ch->table_+64 + 8*next_in + 4);

	    ch->waddr_ = addr;
	    ch->wsize_ = size;
	    ch->woff_  = 0;
      } while (0);
}

void BUS::consume_out_buffer_(Channel*ch)
{
	//XXXX Need to mask i960rp interrupts here?
      unsigned long idx = dev_->readl(ch->table_ + 16);
      idx = (idx + 1) % 4;
      dev_->writel(ch->table_ + 16, idx);

	// Tell the host that something has changed.
      dev_->ring_host(2);
      ch->raddr_ = 0;
      ch->rsize_ = 0;
      ch->roff_  = 0;
}

void BUS::consume_in_buffer_(Channel*ch)
{
	//XXXX Need to mask i960rp interrupts here?
      unsigned long idx = dev_->readl(ch->table_ + 28);

      dev_->writel(ch->table_ + 64 + idx*8 + 4, ch->woff_);
      idx = (idx + 1) % 4;
      dev_->writel(ch->table_ + 28, idx);

      dev_->ring_host(2);
      ch->waddr_ = 0;
      ch->wsize_ = 0;
      ch->woff_  = 0;
}

/*
 * If I get a new root table from the host, I need to read it and get
 * all the channel information out of it for the channels that are
 * available. 
 */
void BUS::set_root_table_ptr_(unsigned long ptr)
{
      if (dev_->readl(ptr) != ROOT_TABLE_MAGIC)
	    ptr = 0;

      if (dev_->readl(ptr+4) != ptr)
	    ptr = 0;

      root_table_ptr_ = ptr;

	      // disconnect;
      if (ptr == 0)
	    return;

      dispatch_sync_.wakeup();
}

void BUS::set_status_message_(unsigned long msg)
{
}

void BUS::set_channel_event_()
{
      dispatch_sync_.wakeup();

      for (Channel*cur = channels_ ;  cur ;  cur = cur->list_)
	    if (cur->readable_fun_)
		  (cur->readable_fun_)();
}


/*
 * $Log: BUS.cc,v $
 * Revision 1.22  1998/05/26 16:16:32  steve
 *  New channel protocol
 *
 */
