/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: region.cc,v 1.2 1997/04/19 05:56:48 steve Exp $"
/*
 * $Log: region.cc,v $
 * Revision 1.2  1997/04/19 05:56:48  steve
 *  copyright notice.
 *
 * Revision 1.1  1997/04/19 05:48:29  steve
 *  Redo the BIOS allocator to be simpler and to
 *  properly handle holes.
 *
 */

# include  <assert.h>
# include  "region.h"

Region::Region(unsigned long a, unsigned long m)
{
      assert( ! (a & ~m) );

      ncells_ = 1;
      cells_[0].addr = a;
      cells_[0].mask = m;
}

/*
 * This function takes the given cell and splits it once, creating a
 * new cell after the current one, making this and the new cell
 * half-size, and pushing everything else back.
 */
void Region::split_(unsigned cur)
{
      assert( ncells_ != (MAX_CELLS-1) );
      for (unsigned idx = ncells_ ;  idx > (cur+1) ;  idx -= 1)
	    cells_[idx] = cells_[idx-1];

      ncells_ += 1;

      unsigned long newmask = (cells_[cur].mask >> 1) | 0x80000000;
      cells_[cur+1].mask = cells_[cur].mask = newmask;

      cells_[cur+1].addr = cells_[cur].addr + ~newmask + 1;
}

/*
 * Remove the identified cell from the table, and pull all the later
 * ones forward.
 */
void Region::remove_(unsigned cur)
{
      ncells_ -= 1;
      for ( ; cur < ncells_ ;  cur += 1)  cells_[cur] = cells_[cur+1];
}

/*
 * This method removes the specified region from the current table. If
 * the region includes space that is not in the table, return false
 * and subtract nothing.
 */
bool Region::subtract(unsigned long addr, unsigned long mask)
{
      assert( ! (addr & ~mask) );

      for (unsigned idx = 0 ;  idx < ncells_ ; idx += 1) {
	      // If this cell is after the region, go on.
	    if (cells_[idx].addr > addr) continue;

	      // If the cell is before or at the region but is too
	      // small, return an error.
	    unsigned long cend = cells_[idx].addr + ~cells_[idx].mask + 1;
	    if (cend < (addr + ~mask + 1)) return false;

	      // If the region is not in the start of the cell, split
	      // the cell and iterate again.
	    if (cells_[idx].addr != addr) {
		  split_(idx);
		  continue;
	    }

	      // At last, the region is at the front of a cell. Split
	      // that cell until it exactly matches the region, then
	      // remove it.
	    while (cells_[idx].mask != mask) split_(idx);
	    remove_(idx);
	    return true;
      }
      return false;
}

unsigned long Region::allocate(unsigned long mask)
{
      if (mask == 0xffffffff) return cells_[0].addr;

      for (unsigned idx = 0 ;  idx < ncells_ ;  idx += 1) {

	      // If this region is too small for what I'm asking for,
	      // (i.e. not enough zeros in the low bits) skip it.
	    if (0 != (cells_[idx].mask & ~mask)) continue;

	    unsigned long raddr = cells_[idx].addr;

	    if (cells_[idx].mask == mask) {
		  remove_(idx);
		  return raddr;
	    }

	      // Split the located cell until it is only twice the
	      // size of the requested region.
	    while (cells_[idx].mask < (mask << 1)) split_(idx);

	      // Now take my piece out of the region.
	    cells_[idx].addr += ~mask + 1;
	    cells_[idx].mask = mask;

	    return raddr;
      }

      return 0;
}
