
/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: GDMA.cc,v 1.6 1996/10/28 03:45:49 steve Exp $"
/*
 * $Log: GDMA.cc,v $
 * Revision 1.6  1996/10/28 03:45:49  steve
 *  License notice.
 *
 * Revision 1.5  1995/11/08 10:08:01  steve
 *  Allow DMA transfers to be stopped.
 *
 * Revision 1.4  1995/10/18 03:11:33  steve
 *  Proper completion synchronization.
 *
 * Revision 1.3  1995/10/17 18:30:19  steve
 *  Abandon the wait_for returning count,
 *  Add test_for() to know when transfers
 *  can be started.
 *
 * Revision 1.2  1995/10/17 16:58:52  steve
 *  Support large GDMA transfers by reloading the
 *  registers when terminal count is reached.
 *
 * Revision 1.1  1995/10/15 21:55:23  steve
 *  Add the GDMA class to manage the GDMA controller.
 *  This takes care of sync and operation.
 *
 */

# include  <mc68322/GDMA.h>
# include  <assert.h>

const unsigned short GDMA_BAD_ADDRESS = 0x0001;
const unsigned short GDMA_COMPLETE    = 0x0002;
const unsigned short GDMA_TERMINAL    = 0x0004;
const unsigned short GDMA_BUSY        = 0x0010;

static bool test_is_zero(const volatile size_t*value)
{ return *value == 0; }


GDMA::GDMA(unsigned char level, unsigned long base)
: regs_( (REGS_*)(base+0x210) ), intr_( (INTR_*)(base+0x740) ),
  csdtr_( (volatile unsigned short*)(base+0x080) )
{
      intr_->enable_flag = 0;
      intr_->level = level;
      regs_->control = 0;
      *csdtr_ = 0;
}

void GDMA::setup_transfer_()
{
      if (total_count_ >= 0x4000)
	active_count_ = 0x3ffe;
      else
	active_count_ = total_count_;

      regs_->dram_address = active_to_;
      regs_->sram_address = (void*)active_from_;
      regs_->count = active_count_;

      intr_->enable_flag = GDMA_BAD_ADDRESS|GDMA_COMPLETE|GDMA_TERMINAL;
}

void GDMA::transfer(void*to, const void*from, size_t count)
{
      assert( ! (intr_->int_flag & GDMA_BUSY) );

      active_to_ = (char*)to;
      active_from_ = from;
      total_count_ = count;
      setup_transfer_();
}

/*
 * XXXX I do not yet support the flush operation.
 */
void GDMA::wait_for(bool flush)
{
        // Wait for the BUSY flag to turn off.
      sync_.sleep(test_is_zero, &total_count_);
}

bool GDMA::test_for() const
{ return total_count_ == 0; }

void GDMA::stop_transfer()
{
      if (! (intr_->int_flag & GDMA_BUSY)) return;

      total_count_ = active_count_;
      regs_->flush = 1;
}

void GDMA::isr()
{
      unsigned short flags = intr_->int_flag;
      intr_->int_flag = flags;
      assert( ! (flags & GDMA_BAD_ADDRESS) );

      if (flags & (GDMA_COMPLETE|GDMA_TERMINAL)) {
	    assert( ! (flags & GDMA_BUSY) );
	    total_count_ -= active_count_;

	    if (total_count_ > 0) {
		  active_to_ += active_count_*2;
		  setup_transfer_();
	    } else {
		  sync_.wakeup();
	    }

      }
}
