/*
 * Copyright (c) 1995 Picture Elements Inc.
 *	Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ParallelPort.cc,v 1.15 1996/10/28 03:45:51 steve Exp $"
/*
 * $Log: ParallelPort.cc,v $
 * Revision 1.15  1996/10/28 03:45:51  steve
 *  License notice.
 *
 * Revision 1.14  1995/11/02 07:30:22  steve
 *  More convenient interface to id string.
 *
 * Revision 1.13  1995/11/02 06:57:16  steve
 *  Split up the ParallelPort source,
 *  Support nibble mode id strings.
 *
 * Revision 1.12  1995/10/16 04:31:14  steve
 *  Fix run too big for a single run,
 *  Handle mode switches close to run lengths.
 *
 * Revision 1.11  1995/10/10 01:58:50  steve
 *  write takes pointer to constant.
 *
 * Revision 1.10  1995/10/08 09:13:20  steve
 *  Combine the sending of run length with the
 *  sending of a non-command byte.
 *
 *  Make reverse_ecp_ properly notice the switch
 *  away from ECP mode.
 *
 *  Only RLE sequences 3 or more bytes long.
 *
 * Revision 1.9  1995/10/08 04:27:54  steve
 *  Get rid of useless member.
 *
 * Revision 1.8  1995/10/05 08:19:50  steve
 *  RLE for ECP reverse mode.
 *
 * Revision 1.7  1995/10/04 23:17:27  steve
 *  Add RLE support to ECP mode.
 *
 * Revision 1.6  1995/10/04 07:17:02  steve
 *  Bum a little more performance out of reverse mode
 *  ECP, and potentially other reverse modes.
 *
 * Revision 1.5  1995/10/04 04:32:43  steve
 *  ECP reverse mode.
 *
 * Revision 1.4  1995/10/03 03:36:10  steve
 *  More 1284 more support.
 *
 * Revision 1.3  1995/09/25 01:38:47  steve
 *  Introduce IEE 1284 support, including nibble
 *  mode and byte mode.
 *
 * Revision 1.2  1995/09/08 23:18:26  steve
 *  change the ParallelPort to read ahead using PDMA.
 *  This should speed things up quite a bit, and allow
 *  the port to run independent of the CPU.
 *
 * Revision 1.1  1995/09/02 18:27:19  steve
 *  Initial import of UCR core to cvs.
 *
 *
 */


# include  <mc68322/ParallelPort.h>
# include  "ParallelPort_priv.h"
# include  <ucr/ucr.h>
# include  <assert.h>


/*
 * This class supports the PPI of the mc68322.
 *
 * The IEEE1284 support terminology (including naming of the pins and
 * numbering of events) comes from the IEEE Std 1284-1994:
 *
 *	IEEE Standard Signalling Method for
 *	a Bidirectional Parallel Peripheral
 *	Interface for Personal Computers.
 */

/*
 * XX READ AHEAD:
 * Once a DMA is interrupted, read-ahead is stopped until the buffer
 * is finished. This is because the PDMA cannot start a transfer on
 * odd boundries but a stopped read-ahead can stop a transfer on an
 * odd byte. When I feel the desire, I should find a way to restart
 * such things when I can.
 */

/*
 * In some cases, I need to delay for .5usec, minimum. One clock is
 * .06usec and 15 make .94usec. The following is thus rather
 * generous. (This may break if a faster clock is used. I should
 * calculate Bogomips, or somesuch.)
 */

void ParallelPort::nap_for_host_()
{
      for (unsigned jiffy = 0 ;  jiffy <= 15 ;  jiffy += 1)
	/* Nowhere fast... */;
}


/*
 * Initially, set up the port in compatibility mode. The device will
 * remain in this mode until the host sets nSelectIn high and nAutoFd
 * low.
 */
ParallelPort::ParallelPort(void*buf, size_t nbuf, unsigned char level,
			   unsigned long base)
: in_flag_(true), buf_((unsigned char*)buf), nbuf_(nbuf),
  transfer_(0), holding_(0), readp_(buf_), dma_ready_(true),
  reverse_transfer_(0), out_remaining_(0), out_buffer_(0), id_(0),
  port_isr_(&port_isr_compat_), rle_flag_(false),
  port_regs_( (REGS_*) (base + 0x300) ),
  port_intr_( (INTR_*) (base + 0x760) ),
  pdma_regs_( (PDMA_*) (base + 0x200) ),
  pdma_intr_( (INTR_*) (base + 0x750) )
{

        // Reset the PDMA controller. (See Section 5.5, page 5.15)
        // Reset the PPI. (See section 9.1.2, page 9-4)
      *(unsigned short*) (base + 0x7a0) = 0x4;
      port_regs_->ppcr = 1;

        // Centronics compatibility mode
      port_regs_->ppcr      = PPCR_COMPAT_MODE|PPCR_FILTER;
      port_regs_->ack_width = 0x000f;
      port_regs_->lines     = PIN_nFault|PIN_Select;

        // Clear any interrupts, and set the level.
      port_intr_->enable_flags = 0x0000;
      port_intr_->intr_flags   = 0xffff;
      port_intr_->level        = level;

      pdma_intr_->enable_flags = 0x0000;
      pdma_intr_->intr_flags   = 0xffff;
      pdma_intr_->level        = level;


        // Set up the isr
      set_isr(level, isr_, this);

        // Enable interrupts
      pdma_intr_->enable_flags = TERMINAL_COUNT|DMA_COMPLETE|ILLEGAL_ADDRESS;
      port_intr_->enable_flags = HIGH_nSelectIn|LOW_nAutoFd;
}

ParallelPort::~ParallelPort()
{
}

bool ParallelPort::test_for_outbound_(ParallelPort*obj)
{
      return obj->reverse_transfer_ != 0;
}

void ParallelPort::set_id_string(unsigned short count, const char*str)
{
      id_ = str;
      nid_= count;
      curid_ = id_;
}

/*
 * Write does its job by shoving bytes into the reverse transfer. The
 * means for doing the transfer is specified by the mode I'm in, but
 * that I get by following a method pointer. The thread that calls
 * this write() blocks as long as the port is in forward mode, or
 * the port is in reverse mode but there are more bytes to transfer.
 */
size_t ParallelPort::write(const void*buffer, size_t count)
{
      unsigned short cookie = thread_splhi();
      out_buffer_ = (unsigned char*)buffer;
      out_remaining_ = count;

      while (out_remaining_ > 0) {
	    out_sync_.sleep(test_for_outbound_, this);
	    void (ParallelPort::*rt)() = reverse_transfer_;

	      // The reverse_transfer_ method runs with interrupts
	      // enabled. It must take into account the possibility
	      // that it is interrupted by a dropped nSelectIn, etc.
	    unsigned short tmpcookie = thread_spl(cookie);
	    (this->*rt)();
	    thread_spl(tmpcookie);
      }

      thread_spl(cookie);
      return count;
}


/*
 * Call this method whenever Event 22 is detected. Manage the
 * handshake to switch back to compatibility mode. Also make sure the
 * state of the ParallelPort object is prepared to cope with the
 * switch to compatibility mode.
 */
void ParallelPort::terminate_ieee1284_()
{
        // Turn OFF transfer wakeups.
      reverse_transfer_ = 0;


        // Event 23: turn off bus drivers, turn on Busy and nFault.
      port_regs_->ppcr = 0|PPCR_FILTER;
      port_regs_->lines = PIN_Busy2|PIN_nFault|port_regs_->lines;

      nap_for_host_();

        // Event 24: nAck is LOW, Select is toggled.
      port_regs_->lines |= PIN_nAck2;
      port_regs_->lines ^= PIN_Select;

      while (port_regs_->lines & PIN_nAutoFd)
	/* Wait for nAutoFd to go low */;

        // Event 25: Host handshake received

        // Event 26: Set pins for compatability mode, busy, nAck still low.
      port_regs_->lines = PIN_Busy2|PIN_nFault|PIN_Select|PIN_nAck2;

      nap_for_host_();

        // Event 27: Complete handshake.
      port_regs_->lines = PIN_Busy2|PIN_nFault|PIN_Select;

      while (0 == (port_regs_->lines & PIN_nAutoFd))
	/* Wait for nAutoFd to go HIGH */;

        // Event 28: Host acknowledges. Let hardware handshaking take
	// over from here, but be ready to switch to 1284 mode again
	// when the lines demand.

      port_regs_->lines = PIN_nFault|PIN_Select;
      port_regs_->ppcr  = PPCR_COMPAT_MODE|PPCR_FILTER;
      port_intr_->enable_flags = HIGH_nSelectIn|LOW_nAutoFd;
      port_isr_ = &port_isr_compat_;

      in_flag_ = true;
      in_sync_.wakeup();
}

/*
 * This interrupt handler notices IEEE1284 Event 1, which starts mode
 * negotiation. The host sets nSelectIn HIGH and nAutoFd low to
 * request  negotiation. I acknowledge by setting a bunch of pins and
 * taking control away from the hardware.
 *
 * The host action constitutes IEEE1284 Event 1. I answer with Event 2.
 */

void ParallelPort::port_isr_compat_()
{

      const unsigned short N_MASK = (PIN_nAutoFd|PIN_nSelectIn);
      if ((port_regs_->lines & N_MASK) == PIN_nSelectIn) {

	      // Event 1: Detect 1284 mode switch

	      // Turn of hardware handshaking and the PDMA controller.
	    in_flag_ = false;
	    port_regs_->ppcr = 0|PPCR_FILTER;
	    pdma_regs_->flush_request = 1;

	      // Event 2: Answer that I'm a 1284 device
	    port_regs_->lines = PIN_nFault|PIN_PError|PIN_Select|PIN_nAck2;
	    port_isr_ = &port_isr_negotiate_;

	      // Set to interrupt on Event 3 from the host.
	    port_intr_->enable_flags = LOW_nStrobe;
      }
}

/*
 * Wait for Event 22 (terminate IEEE1284) then switches back to
 * compatability mode.
 */

void ParallelPort::port_isr_error_()
{
      const unsigned short N_MASK = (PIN_nAutoFd|PIN_nSelectIn);
      if ((port_regs_->lines & N_MASK) == PIN_nAutoFd)
	terminate_ieee1284_();

}

/*
 * Here I notice events 3 and 4. Complete the negotiation and ack any
 * mode that I know I can support. Finally, switch to that mode and
 * start transferring if applicable.
 */

void ParallelPort::port_isr_negotiate_()
{

        // This is event 3...
      if ( (port_regs_->lines & PIN_nStrobe) == 0 ) {
	    port_intr_->enable_flags = HIGH_nStrobe|HIGH_nAutoFd;
      }

        // This is event 4...
      const unsigned short N_MASK = PIN_nStrobe|PIN_nAutoFd;
      if ( (port_regs_->lines & N_MASK) == N_MASK ) {

	      // Acknowledge the modes that I do support. They all
	      // have slightly different protocols from this point, so
	      // switch based on the requested mode.

	    unsigned char request  = port_regs_->data;
	    if (request == 0) { // NIBBLE MODE
		    // Event 5: Set my response, leave nAck low. Set
		    // the nFault pin to indicate that I have no data
		    // available.
		  if (out_remaining_ > 0)
		    port_regs_->lines = PIN_nAck2;
		  else
		    port_regs_->lines = PIN_nFault|PIN_PError|PIN_nAck2;

		  nap_for_host_();

		    // Event 6: Raise nAck to latch response in host.
		  port_regs_->lines &= ~PIN_nAck2;
		  enter_nibble_mode_();

	    } else  switch (request) {

		case 0x01: // BYTE MODE
		    // Event 5: Set my response, leave nAck low. Set
		    // the nFault pin to indicate that I have no data
		    // available.
		  if (out_remaining_ > 0)
		    port_regs_->lines = PIN_Select|PIN_nAck2;
		  else
		    port_regs_->lines = PIN_Select
		                       |PIN_nFault
				       |PIN_PError
				       |PIN_nAck2;

		  nap_for_host_();

		    // Event 6: Raise nAck to latch response in host.
		  port_regs_->lines &= ~PIN_nAck2;
		  enter_byte_mode_();
		  break;

		case 0x10: // ECP MODE
		case 0x30: // ECP MODE with RLE

		    // Event 5: Set my response, leave nAck low.
		  port_regs_->lines = PIN_Select|PIN_nAck2;
		  nap_for_host_();

		    // Event 6: Raise nAck to latch response in host.
		  port_regs_->lines = PIN_Select|PIN_nFault;

		  rle_flag_ = request == 0x30;
		  enter_ecp_mode_();
		  break;

		case 0x04: // NIBBLE ID MODE
		  assert(id_);
		  port_regs_->lines = PIN_Select
		                     |PIN_nFault
				     |PIN_PError
				     |PIN_nAck2;

		  nap_for_host_();

		    // Event 6: Raise nAck to latch response in host.
		  port_regs_->lines &= ~PIN_nAck2;
		  enter_nibble_id_mode_();
		  break;

		default: // INVALID MODE.

		    // Event 5: Set Select low to reject request.
		  port_regs_->lines = PIN_nAck2;
		  nap_for_host_();

		    // Event 6: Raise nAck to latch response in host.
		  port_regs_->lines &= ~PIN_nAck2;

		  port_intr_->enable_flags = LOW_nSelectIn|HIGH_nAutoFd;
		  port_isr_ = &port_isr_error_;
	    }
      }
}
