/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: fopen.cc,v 1.4 1996/10/28 03:46:02 steve Exp $"
/*
 * $Log: fopen.cc,v $
 * Revision 1.4  1996/10/28 03:46:02  steve
 *  License notice.
 *
 * Revision 1.3  1996/09/19 01:38:34  steve
 *  Add fgetc and fgets. stdio input works.
 *
 * Revision 1.2  1996/09/16 17:03:07  steve
 *  Add the read/write state machine.
 *
 * Revision 1.1  1996/09/14 00:10:19  steve
 *  Add a stdio library.
 *
 */

# include  "stdiox.h"
# include  <string.h>
# include  <assert.h>

/*
 * Implement the ISO/ANSI C fopen function. This code parses the
 * filename and mode strings, forms a FILE structure, and makes sure
 * the selected device exists and can cope with the details. If
 * everything checks out, return a FILE*, otherwise return 0.
 */

extern "C" FILE * fopen(const char*filename, const char*mode)
{

	// Parse the filename into the device type and instance
	// name. The device type is the part before the colon, and the
	// instance name is the part after. If there is no instance
	// part, then the colon is optional. After the parsing is
	// done, the devtype holds the device type, and the cop
	// pointer points to the instance part.

      const char*cop = strchr(filename, ':');
      if (cop == 0) cop = filename + strlen(filename);

      char devtype[8];
      strncpy(devtype, filename, cop-filename);
      devtype[cop-filename] = 0;

      if (*cop == ':') cop += 1;

	// Use the parsed out device type to find a DeviceType object
	// that represents the device at hand.

      StdioDeviceType* type = StdioDeviceType::find(devtype);
      if (type == 0) return 0;

      StdioDevice *dev = type->open(cop);
      if (dev == 0) return 0;

	// Go through the mode string, converting the letters to mode
	// flag bits. As soon as I get an unexpected character
	// (including a nul) finish the processing of the mode.

      unsigned mode_flags = 0;
      switch (mode[0]) {
	  case 'r':
	    mode_flags |= _FILE::F_READ;
	    break;
	  case 'w':
	    mode_flags |= _FILE::F_WRITE;
	    break;
	  case 'a':
	    mode_flags |= _FILE::F_WRITE;
	    break;
	  default:
	    goto done_mode;
      }

      for (mode += 1;;mode += 1) switch (*mode) {

	      // Binary? This is not DOS, ignore this flag!
	  case 'b':
	    break;

	      // The + flag causes the stream to be
	      // read/write. The difference between r+, w+ and
	      // a+ involve the file pointer and whether to
	      // truncate on startup.
	  case '+':
	    mode_flags |= _FILE::F_READ;
	    mode_flags |= _FILE::F_WRITE;
	    break;

	  default:
	    goto done_mode;
      }
done_mode:

	// Check the calculated mode bits with the capabilities of the
	// device, to make sure what's been asked of us is really
	// possible.

      if ( (mode_flags&_FILE::F_WRITE) && (!dev->writeable()) ) {
	    type->close(dev);
	    return 0;
      }

      if ( (mode_flags&_FILE::F_READ) && !dev->readable() ) {
	    type->close(dev);
	    return 0;
      }

      FILE*result = new _FILE;
      result->type = type;
      result->dev = dev;

      result->flags = mode_flags;
      result->state = _FILE::S_IDLE;

	// Line buffered if the device is interactive, fully buffered
	// otherwise. (XXXX What about read-only devices?)

      if (dev->interactive()) result->buf_mode = _IOLBF;
      else result->buf_mode = _IOFBF;

      result->buf_len  = BUFSIZ;
      result->buf = new char[BUFSIZ];
      result->buf_ptr = result->buf;
      result->buf_freeme = true;

      return result;
}
