#ifndef __ucrif_H
#define __ucrif_H
/*
 * Copyright (c) 1996,1997,1998 Picture Elements, Inc.
 *    Stephen Williams <steve@picturel.com>
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#if !defined(WINNT)
#ident "$Id: ucrif.h,v 1.3 1998/06/06 05:40:04 steve Exp $"
#endif
/*
 * This is the interface for the ucr channel driver.
 *
 * Each device supports up to 256 channels, from 0 to 0xff. Each
 * channel may be opened by no more then one process at a time. Once a
 * process opens a channel, reads and writes cause data to move
 * between the host and the target board, through the selected
 * channel. A process can change the channel association by using the
 * UCR_CHANNEL ioctl.
 *
 * Normally, the driver will buffer writes to the board up to a page
 * size, and may have a few buffers filled before a write will
 * block. Also, the application may read bytes from the channel
 * without affecting the write buffer. However, if a read blocks on a
 * channel, the write buffers will be flushed. This fits well with the
 * request/response nature of typical communications with the
 * board. In general, all will work transparently.
 *
 * The application may explicitly flush the write buffer, thus
 * guaranteeing that all written bytes are delivered to the target
 * board, by issuing the UCR_FLUSH ioctl.
 *
 * A UCR_FLUSH does *not* guarantee that all the data has been read by
 * the target -- only that the target has been told about it. UCR_SYNC
 * will block until the target board reads all the data in the channel.
 *
 * Reads from a channel will not block unless there are no bytes
 * ready. In other words, if you read N bytes, the read may actually
 * read 1 to N bytes. The read return code tells you what you have
 * got. If non-blocking mode is enabled (by fcntl under linux or the
 * UCR_NONBLOCK under NT) read will return 0 bytes if there are no
 * bytes ready to be read.
 *
 * Finally, closing a channel causes the channel and host buffer
 * resources to be removed. The close does NOT flush the buffers. It
 * is recommended that a UCR_SYNC preceed a close to prevend a loss of
 * output data.
 *
 * NOTE: Channel 0 is special. When a process first opens a ucr
 * device, it attempts to connect to channel 0. If another process has
 * connected to channel 0, the open will fail and return EBUSY. The
 * open will succeed only if channel 0 is opened by no one else.
 *
 * NOTE: When no process has the device open, the driver completely
 * detaches from the board. This allows the board to be reset without
 * confusing the driver.
 *
 * NOTE ON SELECT: The driver does in fact support the select system
 * call under Linux, but for input data only. NT does not have a
 * general purpose select system call, use overlapped IO instead.
 *
 * NOTE ON Windows NT: The IOCTL codes are compatible with Windows NT
 * encoding using BUFFERED I/O of the parameters. The FILE_DEVICE type
 * bits and the rw bits are also consistent with the use by the driver
 * under NT. Linux ignores the bit patterns, so there is no
 * portability issue.
 */

# define UCR_READFLAG  0x4000
# define UCR_WRITEFLAG 0x8000
# define UCR_(rw,x)  (0x80010000+(rw)+((x)<<2))
# define UCRX_(rw,x) (0x80030000+(rw)+((x)<<2))

/*
 * The driver is free to buffer bytes being sent to a channel. This
 * allows the driver to send larger bursts. The driver will certainly
 * flush on close and channel changes, but sometimes it is necessary
 * to send buffered data immediately.
 *
 * SYNC is more thorough then FLUSH. After a sync, not only is the
 * buffer flushed to the target, but the target has actually READ the
 * data.
 */
# define UCR_FLUSH UCR_(0,0)
# define UCR_SYNC UCR_(0,12)

/*
 * Use this ioctl to set the channel used for reads and writes. The
 * value is 0 when the device is first opened, and can be any number
 * from 0 to 255 (0xff).
 */
# define UCR_CHANNEL UCR_(0,1)

/*
 * Use this to send an attention signal to the uCR target. The single
 * argument is sent to the target as an argument to the attention
 * handler for the channel. (The channel used is set by the
 * UCR_CHANNEL ioctl above.)
 *
 * The target application channel sees attentions delivered this way
 * with the arguments arg1==1 and arg2 from the argument of the ioctl.
 */
# define UCR_ATTENTION UCR_(UCR_WRITEFLAG,2)

/*
 * Use this to send a FILE_MARK to the channel on the uCR target. The
 * target will read this as an end-of-file so generally this is
 * considered the end of the data stream.
 *
 * However, the target is capable of turning OFF the end-of-file, so
 * it can also be used to separate a stream of files.
 */
# define UCR_SEND_FILE_MARK UCR_(UCR_WRITEFLAG,3)

/*
 * The driver can manage up to 16 frames, each no larger then
 * 256Meg. Use UCR_MAKE_FRAME to create a frame with the given size.
 * The argument is of the form 0xfsssssss where f is the frame
 * identifier and s is the size, in bytes.
 *
 * If the kernel cannot allocate the frame, the ioctl return
 * ENOMEM. If the frame already exists, it returns the size of the
 * existing frame. If the existing frame is too small, it can only be
 * made bigger by freeing it and remaking it.
 *
 * UCR_FREE_FRAME takes the same format and releases the frame from
 * the kernel memory. The size bits are ignored.
 *
 * Once the frame exists on the host, the host and board can access
 * the frame independent of any channels. The host can map the frame
 * and close its channels. The target can read/write the frame as long
 * as it exists.
 *
 *  - Persistence Of Frames -
 * On the host, the frames last until the module is unloaded (possibly
 * automaticly) from the kernel, or the frame is explicitly freed. As
 * long as the frame is not freed, any data in it is preserved.
 *
 * On Linux, modules are automatically unloaded if there are no
 * processes with opened channels or mapped frames. The operating
 * system automatically notices that the driver has been idle for a
 * while and arranges for it to be unloaded.
 *
 * NT does not support such a facility, so frames will persist either
 * until the driver module is unloaded manually, or the frame is
 * released explicitly.
 *
 *  - NOTE wrt UCR_SEND_FRAME -
 * UCR_SEND_FRAME is obsolete. It is no longer implemented in the
 * driver, and should not be used.
 */
# define UCR_MAKE_FRAME UCR_(0,4)
# define UCR_FREE_FRAME UCR_(0,5)
# define UCR_SEND_FRAME UCR_(0,6)


/*
 * Return the number of bytes ready to be read from the channel. At
 * least this many bytes can be read, maybe more by the time you get
 * to it. This call does not block. Under Linux, the call would look
 * like this:
 *
 *    int count = ioctl(fd, UCR_NREAD, 0);
 *
 * The count gets loaded with the size, or -1 if there is an error.
 */
# define UCR_NREAD UCR_(0,7)

#if defined(WINNT) || defined(WIN32)
/*
 * This is Windows NT support for mapping a frame. Windows does not
 * have a mmap call, so the driver supports a device specific ioctl
 * that does the job.
 *
 * Both the MMAP and MUNMAP take as input a UcrMmapInfo structure. For
 * mapping, the off and size are used to select the location within
 * the area. The offset must be on a page boundry. The MMAP also
 * returns a structure with all the fields, including the base, filled
 * in. If the size is passed as 0, the entire frame is mapped and the
 * size will be filled in by the driver.
 *
 * For MUNMAP, the driver unmaps the region at the base, and returns
 * nothing as a result.
 */
struct UcrMmapInfo {
      unsigned frame_id;
      unsigned long off;
      unsigned long size;
      void*base;
};
# define UCR_MMAP_FRAME UCR_(UCR_READFLAG|UCR_WRITEFLAG, 8)
# define UCR_MUNMAP_FRAME UCR_(UCR_READFLAG|UCR_WRITEFLAG, 9)
#endif

#if defined(WINNT) || defined(WIN32)
/*
 * This is windows NT support for setting the channel reader to
 * non-blocking mode. What happens normally, is a read with no bytes
 * available will block until at least one byte can be
 * returned. ReadFile won't return 0 bytes. However, this IOCTL can be
 * used to turn on non-blocking mode. In that case, ReadFile will
 * return 0 if there is nothing to read.
 *
 * This ioctl takes a single long parameter, set to 1 to turn on
 * non-blocking mode.
 */
# define UCR_NONBLOCK UCR_(0, 10)
#endif

/*
 * This IOCTL takes to parameters and returns no result. It is for use
 * during testing, the prod the device driver.
 */
# define UCR_TEST UCR_(0, 11)

/*
 * The uCR includes a command/control device separate from the normal
 * device. These commands are often board/os specific.
 */

/*
 * Reset the board. This causes any boot operations on the target
 * board to happen.
 *
 * No channels may be open when this is called.
 */
# define UCRX_RESET_BOARD UCRX_(0,0)

# define UCRX_RUN_PROGRAM UCRX_(0,1)

# define UCRX_DIAGNOSE UCRX_(0,2)

/*
 * This sets the driver trace bits.
 */
# define UCRX_SET_TRACE UCRX_(0,3)
# define UCRX_GET_TRACE UCRX_(0,4)
# define UCR_TRACE_CHAN  0x0001
# define UCR_TRACE_FRAME 0x0002
# define UCR_TRACE_PROTO 0x0004
# define UCR_TRACE_UCRX  0x0008

/*
 * $Log: ucrif.h,v $
 * Revision 1.3  1998/06/06 05:40:04  steve
 *  Linux/alpha 2.1.100 port.
 *
 * Revision 1.2  1998/05/29 18:11:22  steve
 *  Proper cancel/abort behavior for NT.
 *
 * Revision 1.1  1998/05/26 16:16:35  steve
 *  New channel protocol
 *
 * Revision 1.23  1998/04/13 20:48:48  steve
 *  Get debug flags by ioctl.
 *
 * Revision 1.22  1998/04/07 22:06:12  steve
 *  More tracing support.
 *
 * Revision 1.21  1998/04/03 01:35:13  steve
 *  Add assertion support.
 *
 * Revision 1.20  1998/04/01 00:23:06  steve
 *  Support partial reads.
 *
 * Revision 1.19  1998/03/23 19:54:22  steve
 *  More careful testing.
 *
 * Revision 1.18  1998/03/23 03:30:25  steve
 *  non-blocking reading under NT.
 *
 * Revision 1.17  1998/03/23 00:07:38  steve
 *  initial frame support for winnt.
 *
 * Revision 1.16  1998/03/11 00:07:17  steve
 *  Support non-blocking input, and add an ioctl
 *  that tests for input data.
 *
 * Revision 1.15  1998/01/29 22:00:07  steve
 *  Flush writes when read blocks.
 *
 * Revision 1.14  1997/12/07 02:16:43  steve
 *  Fix NT blocking problems, redo the console to
 *  not block and keep a limited buffer, trace the
 *  ucrx operations, and ucrx operations work on NT.
 *
 * Revision 1.13  1997/12/02 02:29:04  steve
 *  Add support for ISE board reset.
 *
 * Revision 1.12  1997/11/11 11:19:46  steve
 *  Add support for frames.
 *
 * Revision 1.11  1997/09/11 21:58:36  steve
 *  Port the uCR device driver to Windows NT,
 *  make the configure select the target board
 *  and target host interface headers.
 *
 * Revision 1.10  1997/08/18 21:53:46  steve
 *  Support inline channel commands.
 *
 * Revision 1.9  1997/06/16 05:14:50  steve
 *  New channel protocol design and implementation.
 *
 * Revision 1.8  1997/03/06 23:27:31  steve
 *  Add specific trace bits for ucr driver tracing,
 *  Simplify the frame allocation to use only pages.
 *
 * Revision 1.7  1997/03/05 03:34:36  steve
 *  Fix major bugs in frame handling.
 *
 * Revision 1.6  1997/03/03 22:00:48  steve
 *  Reuse frame if possible.
 *
 * Revision 1.5  1997/02/28 03:28:22  steve
 *  Add to the ucr linux driver support for a frame buffer,
 *  and add to the BUS class support for is as well.
 *
 *  The linux driver uses GNU style autoconf for configuration
 *  and install.
 *
 * Revision 1.4  1996/10/28 03:46:58  steve
 *  License notice.
 *
 * Revision 1.3  1996/10/09 18:37:34  steve
 *  The ucr driver now prevents file descriptors opening
 *  opened channels. Document this fact.
 *
 *  Polish documentation for release, and add copyright
 *  notices that were missing.
 *
 * Revision 1.2  1996/09/08 17:12:30  steve
 *  Add attention capability.
 *
 * Revision 1.1  1996/09/03 23:43:07  steve
 *  Add the cyclone linux driver.
 *
 */


#endif
