/*
 * Copyright (c) 1996 Picture Elements, Inc.
 *    Stephen Williams
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    Library General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this program; if not, write to the Free
 *    Software Foundation, Inc.,
 *    59 Temple Place - Suite 330
 *    Boston, MA 02111-1307, USA
 *
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc.,
 *    777 Panoramic Way
 *    Berkeley, CA 94704.
 */
#ident "$Id: ucr.c,v 1.4 1997/08/14 05:42:28 steve Exp $"
/*
 * $Log: ucr.c,v $
 * Revision 1.4  1997/08/14 05:42:28  steve
 *  Fix bug in debugging messages.
 *
 * Revision 1.3  1997/08/14 04:52:04  steve
 *  Fix handling of multiple interrupts.
 *
 * Revision 1.2  1996/10/28 03:46:49  steve
 *  License notice.
 *
 * Revision 1.1  1996/09/10 00:45:26  steve
 *  Add gdb interface to project.
 *
 */

/*
 * This is the principle driver for the uCR interface for GDB. It
 * uses the ucr driver interface and the gdb_if protocol for
 * communicating with a proxy on the hardware.
 */

#include "defs.h"
#include "gdbcore.h"
#include "target.h"
#include "inferior.h"
#include "thread.h"

# include <fcntl.h>
# include "ucrif.h"
# include "gdb_if.h"

/* This is the fd of the opened uCR board. */
static int remote_fd = -1;

/* Use this number to sequence command and response between the
   debugger (me) and the target device. The device will include the
   sequence number in any response it sends back. I can then use that
   to detect synchronization troubles. */
static unsigned long sequence = 0;


static void ucr_open(char*, int);
static void ucr_close(int);
static void ucr_detach(char*, int);
static void ucr_resume(int, int, enum target_signal);
static int  ucr_wait(int, struct target_waitstatus *);
static void ucr_fetch_registers(int regno);
static void ucr_store_registers(int regno);
static void ucr_prepare_to_store(void);
static int  ucr_xfer_memory(CORE_ADDR memaddr, char *myaddr, int len,
			     int write, struct target_ops * target);
static void ucr_files_info(struct target_ops *ops);
static int  ucr_insert_breakpoint(CORE_ADDR, char *);
static int  ucr_remove_breakpoint(CORE_ADDR, char *);
static void ucr_kill(void);
static void ucr_mourn_inferior(void);
static int  ucr_can_run(void);
static int  ucr_thread_alive(int pid);


static struct target_ops ucr_ops = {
      "ucr",
      "Picture Elements uCR/i960",
      "Debug on an Intel i960 target running uCR and accessed via PCI.\n"
      "Specify the device node of the board to debug.",

      ucr_open,
      ucr_close,
      0, /* to_attach */
      ucr_detach,
      ucr_resume,
      ucr_wait,
      ucr_fetch_registers,
      ucr_store_registers,
      ucr_prepare_to_store,
      ucr_xfer_memory,
      ucr_files_info, /* to_files_info */
      ucr_insert_breakpoint, /* to_insert_breakpoint */
      ucr_remove_breakpoint, /* to_remove_breakpoint */

      0,
      0,
      0,
      0,
      0,

      ucr_kill,
      0, /* to_load */
      0, /* to_lookup_symbol */

      0, /* to_create_inferior */
      ucr_mourn_inferior,
      ucr_can_run,
      0, /* to_notice_signals */
      ucr_thread_alive,
      0,

      process_stratum,
      0,

      1, /* to_has_all_memory */
      1, /* to_has_memory */
      1, /* to_has_stack */
      1, /* to_has_registers */
      1, /* to_has_execution */

      0,
      0,

      OPS_MAGIC
};

static const char*const cmd_names[] = {
      "no-op",
      "memory-write",
      "memory-read",
      "get-threads",
      "get-registers",
      "set-registers",
      "thread-test",
      "wait",
      "resume",
      "set-break",
      "clear-break",
      0
};

static void print_msg_data(const char*prefix, struct gdb_if*msg, void*data)
{
      unsigned idx;
      unsigned long*dp = (unsigned long*)data;

      printf_unfiltered("%s message #%u cmd=%u[%s] ext=%u "
			"args=(%x %x %x %x)\n", prefix, msg->sequence,
			msg->cmd_code, cmd_names[msg->cmd_code],
			msg->ext, msg->arg1, msg->arg2, msg->arg3,
			msg->arg4);

      if ((msg->ext == 0) || (remote_debug < 2) || (data == 0)) return;

      for (idx = 0 ;  idx < msg->ext ;  idx += 1) {
	    if (idx%4 == 0) printf_unfiltered("       ");
	    printf_unfiltered(" %08x", *dp++);
	    if (idx%4 == 3) printf_unfiltered("\n");
      }

      if (idx%4 != 0) printf_unfiltered("\n");
}

static void send_message(unsigned char cmd, unsigned long arg1,
			 unsigned long arg2, unsigned long arg3,
			 void*data, size_t ndata)
{
      struct gdb_if msg;

      msg.magic1 = GDB_IF_MAGIC1;
      msg.sequence = ++sequence;
      msg.cmd_code = cmd;
      msg.ack = 0;
      msg.ext = (ndata+3) / 4;
      msg.arg1 = arg1;
      msg.arg2 = arg2;
      msg.arg3 = arg3;
      msg.arg4 = 0;
      msg.magic2 = GDB_IF_MAGIC2;

      if (remote_debug > 1) print_msg_data("Send", &msg, data);

      write(remote_fd, &msg, sizeof msg);
      if (msg.ext) write(remote_fd, data, msg.ext*4);
      ioctl(remote_fd, UCR_FLUSH);
}

static int recv_message(struct gdb_if*head, void*data, size_t ndata)
{
      int rc;

      memset(head, 0, sizeof *head);
      rc = read(remote_fd, head, sizeof *head);
      if ((rc == -1) && (errno == EINTR)) return -1;

      if (rc != sizeof *head)
	    error("ucr: Bad read: expect %d, got %d", sizeof *head, rc);

      if (head->magic1 != GDB_IF_MAGIC1)
	    error("ucr: Bad magic(1) from target.");
      if (head->magic2 != GDB_IF_MAGIC2)
	    error("ucr: Bad magic(2) from target.");
      if (head->sequence != sequence)
	    error("ucr: Sequence number mismatch");

      if (head->ext == 0) {
	    if (remote_debug > 0) print_msg_data("Recv", head, 0);
	    return;
      }

      if (ndata < head->ext*4) {
	    void*buffer = malloc(head->ext*4);
	    rc = read(remote_fd, buffer, head->ext*4);
	    memcpy(data, buffer, ndata);
	    free(buffer);
      } else {
	    rc = read(remote_fd, data, head->ext*4);
      }

      if (rc != head->ext*4)
	    error("ucr: Bad read: expect %d, got %d", sizeof head->ext*4, rc);

      if (remote_debug > 0) print_msg_data("Recv", head, data);

      return 0;
}

static enum target_signal ucr960_fault_to_signal[16] =
{
      TARGET_SIGNAL_BUS, /* PARALLEL/OVERRIDE */
      TARGET_SIGNAL_TRAP, /* TRACE */
      TARGET_SIGNAL_ILL, /* OPERATION */
      TARGET_SIGNAL_FPE, /* ARITHMETIC */
      TARGET_SIGNAL_FPE, /* FLOATING POINT */
      TARGET_SIGNAL_ILL, /* CONSTRAINT */
      TARGET_SIGNAL_SEGV, /* memory map fault */
      TARGET_SIGNAL_SEGV, /* PROTECTION */
      TARGET_SIGNAL_BUS,
      TARGET_SIGNAL_BUS,
      TARGET_SIGNAL_ILL, /* TYPE */
      TARGET_SIGNAL_UNKNOWN,
      TARGET_SIGNAL_UNKNOWN,
      TARGET_SIGNAL_UNKNOWN,
      TARGET_SIGNAL_UNKNOWN,
      TARGET_SIGNAL_UNKNOWN
};

/*
 * Open the device and connect to the correct channel for talking with
 * the debug proxy. Make sure the target is alive by sending a NOOP
 * command and getting the response.
 */
static void ucr_open(char*name, int from_tty)
{
      unsigned long buf[16];
      unsigned idx;
      struct gdb_if head;
      int rc;

      if (name == 0) error("Please specify the target device file"
			   " (e.g. /dev/cyclone0");

      remote_fd = open(name, O_RDWR, 0);
      if (remote_fd == -1) error("Error (%s) opening the target.",
				 safe_strerror(errno));


	/* The uCR channel 255 is reserved for the debugger. Use this
	   for communicating with the proxy on the target. */
      rc = ioctl(remote_fd, UCR_CHANNEL, 255);
      if (rc == -1) error("Error (%s) setting channel.",
			  safe_strerror(errno));


      push_target(&ucr_ops);

	/* Get from the target the ID of a thread that is faulty, or
	   the stopped one at any rate. By passing 1 in arg1 of WAIT
	   I'm telling the target to stop now no matter what. */

      send_message(GIF_CMD_WAIT, 1, 0, 0, 0, 0);
      recv_message(&head, 0, 0);
      inferior_pid = head.arg1;
      attach_flag = 1;
      select_frame(get_current_frame(), 0);


	/* Now that the tsrget is stopped and we're attached, get all
	   the threads and announce them to the debugger. */

      init_thread_list();

      send_message(GIF_CMD_THREADS, 0, 0, 0, 0, 0);
      recv_message(&head, buf, sizeof buf);
      for (idx = 1 ;  idx < head.arg1 ;  idx += 1)
	    add_thread(buf[idx]);

}

/*
 * Close completely disconnects me from the target device. After this,
 * I no longer have any business with the target.
 */
static void ucr_close(int quitting)
{
	/* Final cleanup. This closes the connection to the uCR
	   device. */
      close(remote_fd);
      remote_fd = -1;
}


/*
 * With the uCR target, typing "detach" to gdb causes the connection
 * to the uCR device be completely disconnected. I manage this by
 * popping myself from the target stack, which will implicitly call
 * the target_close (and thus ucr_close) method.
 */
static void ucr_detach(char*args, int from_tty)
{
      if (from_tty) puts_unfiltered("Closing uCR debug channel.\n");

      {
	    struct gdb_if head;
	    send_message(GIF_CMD_RESUME, -1, 0, 0, 0, 0);
	    recv_message(&head, 0, 0);
      }

	/* By clearing the attach_flag and inferior_pid variables, I
	   am telling gdb that I am not attached, and there is
	   certainly no current thread. */
      attach_flag = 0;
      inferior_pid = 0;
      init_thread_list();
      flush_cached_frames();


	/* Remove the target from the stack. That is how this one
	   works... it is either attached or closed. */
      unpush_target(&ucr_ops);
}

/*
 * Rather then resume all the threads, I resume the thread of current
 * interest, and all the unfaulted threads. The wait some time later
 * will be called, and if this resume left a faulted thread behind it
 * will be noticed then. Through this algorithm, one can cycle through
 * all the faulty threads.
 */
static void ucr_resume(int pid, int step_flag, enum target_signal sig)
{
      struct gdb_if head;

      if (remote_debug)
	    printf_filtered("ucr %s: resume(%d, %d, %d)\n",
			    ucr_target_pid_to_str(inferior_pid), pid,
			    step_flag, sig);

      if (pid != -1)
	    printf_unfiltered("XXXX resume(%d, %d, %d)"
			      " not supported.\n",
			      pid, step_flag, sig);


      {
	    unsigned long value = (step_flag? 2 : 0);
	    target_prepare_to_store();
	    memcpy(&registers[REGISTER_BYTE (34)], &value, sizeof value);
	    target_store_registers(34);
      }

      if (sig)
	    printf_unfiltered("XXXX resume(%d, %d, %d)"
			      " sending signals not supported.\n",
			      pid, step_flag, sig);

      if (inferior_pid) {
	    send_message(GIF_CMD_RESUME, inferior_pid, 0, 0, 0, 0);
	    recv_message(&head, 0, 0);
      }
      send_message(GIF_CMD_RESUME, 0, 0, 0, 0, 0);
      recv_message(&head, 0, 0);
      registers_changed();
}

/*
 * The target_wait method tells the remote to send me back an
 * interesting event. If none is forthcomming, this method will wait
 * until one does arrive. I fill in the status with the event that
 * occurred, and return the pid (thread id in my case) of the thread
 * that it happened to.
 */
static int  ucr_wait(int pid, struct target_waitstatus *status)
{
      int rc;
      struct gdb_if head;
      if (remote_debug || (pid != -1))
	    printf_unfiltered("ucr %s: ucr_wait(0x%08x, ...)\n",
			      ucr_target_pid_to_str(inferior_pid), pid);

	/* Send the wait request to the target, then receive the
	   response. The target may already be stopped, or I may have
	   to wait for the reply to wait. No problem. */

      send_message(GIF_CMD_WAIT, 0, 0, 0, 0, 0);
      rc = recv_message(&head, 0, 0);
      if (rc == -1) {

	      /* If I woke up without getting a reply, send an
		 attention to the target and try again to receive the
		 wait reply. */
	    if (remote_debug)
		  printf_unfiltered("ucr: error waiting for target."
				    " Send an attention.\n");

	    rc = ioctl(remote_fd, UCR_ATTENTION, 1);
	    rc = recv_message(&head, 0, 0);
	    if (rc == -1) error("error waiting for target.");
      }


	/* XXXX For now, only STOP BY SIGNAL is supported. This target
	   will never support much else, but abort will surely be
	   added. */

      status->kind = TARGET_WAITKIND_STOPPED;
      if (head.arg2 == 0xffffffff)
	    status->value.sig = TARGET_SIGNAL_STOP;
      else
	    status->value.sig = ucr960_fault_to_signal[head.arg2];


	/* The target returns the thread id in arg1, and the fault
	   code in arg2. GDB uses this to know to whom the event
	   happened. */
      return head.arg1;
}

/*
 * This method pulls registers for a specific thread out of the target
 * and stores them in the GDB register cache. Registers that to not
 * exist are set to zero.
 */
static void ucr_fetch_registers(int regno)
{
      int idx;
      struct gdb_if head;
      char regs[REGISTER_BYTES];

      memset(regs, 0, REGISTER_BYTES);
      send_message(GIF_CMD_GET_REGS, inferior_pid, 0, 0, 0, 0);
      recv_message(&head, regs, sizeof regs);

      for (idx = 0 ;  idx < NUM_REGS ;  idx += 1) {
	    supply_register(idx, regs+REGISTER_BYTE(idx));
      }

}

static void ucr_store_registers(int regno)
{
      struct gdb_if head;
      unsigned long val = *(unsigned long*)(registers+REGISTER_BYTE(regno));
      if (regno < 0) error("XXXX ucr_store_registers(-1) not yet supported.");

      if (remote_debug)
	    printf_unfiltered("ucr %s: Write %d(0x%x) into register %s\n",
			      target_pid_to_str(inferior_pid), val,
			      val, reg_names[regno]);

      send_message(GIF_CMD_SET_REGS, inferior_pid, regno, val, 0, 0);
      recv_message(&head, 0, 0);
}

static void ucr_prepare_to_store(void)
{
}


/*
 * The debugger uses this method to access core memory of the
 * target. It uses this to write or read bytes. I actually perform
 * these feats by sending commands with the address and length of the
 * memory, and receive back acknowledgement that the operation is done
 * and if a read, the bytes read.
 *
 * XXXX I do not yet support getting error codes back. I must trust
 * that the data I write has actually been written.
 */
static int  ucr_xfer_memory(CORE_ADDR memaddr, char *myaddr, int len,
			     int write_flag, struct target_ops * target)
{
      if (remote_debug)
	    printf_unfiltered("ucr: %s %d bytes at address 0x%08x\n",
			      write_flag? "Write" : "Read", len,
			      memaddr);

      if (write_flag) {
	    struct gdb_if head;
	    send_message(GIF_CMD_WRITE, memaddr, len, 0, myaddr, len);
	    recv_message(&head, 0, 0);
      } else {
	    struct gdb_if head;
	    send_message(GIF_CMD_READ, memaddr, len, 0, 0, 0);
	    recv_message(&head, myaddr, len);
      }

      return len;
}

static int ucr_insert_breakpoint(addr, contents_cache)
     CORE_ADDR addr;
     char *contents_cache;
{
      struct gdb_if head;

      if (remote_debug)
	    printf_unfiltered("ucr: Insert breakpoint at 0x%0x\n", addr);

      send_message(GIF_CMD_SET_BREAK, addr, 0, 0, 0, 0);
      recv_message(&head, contents_cache, BREAKPOINT_MAX);
      return 0;
}

static int ucr_remove_breakpoint (addr, contents_cache)
     CORE_ADDR addr;
     char *contents_cache;
{
      struct gdb_if head;

      if (remote_debug)
	    printf_unfiltered("ucr: Remove breakpoint from 0x%0x\n", addr);

      send_message(GIF_CMD_CLR_BREAK, addr, 0, 0, contents_cache,
		   BREAKPOINT_MAX);
      recv_message(&head, 0, 0);
      return 0;
}

static void ucr_files_info(struct target_ops*ignore)
{
      puts_filtered("Debugging a uCR application.\n");
}

static void ucr_kill(void)
{
      error("XXXX ucr_kill not yet supported.");
}

static void ucr_mourn_inferior(void)
{
      error("XXXX ucr_mourn_inferior not yet supported.\n");
}

/*
 * GDB uses this method to tell if the target is capable of being
 * executed. In uCR, this is always the case.
 */
static int  ucr_can_run()
{
      return (remote_fd >= 0) ? 1 : 0;
}

/*
 * GDB uses this method to test that a thread is alive. I should check
 * with the target application that it is.
 */
static int  ucr_thread_alive(int pid)
{
      struct gdb_if head;
      send_message(GIF_CMD_THREAD_TEST, pid, 0, 0, 0, 0);
      recv_message(&head, 0, 0);
      return head.arg1;
}


char*ucr_target_pid_to_str(int pid)
{
      static char buffer[] = "thread 0x12345678";
      sprintf(buffer, "thread 0x%08x", pid);
      return buffer;
}

/*
 * This is the magic init routine that GDB calls to link me in. There
 * is precious little to do here.
 */
void
_initialize_ucr ()
{
      add_target (&ucr_ops);
}
