#ifndef __ucrpriv_H
#define __ucrpriv_H
/*
 * Copyright (c) 1997 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#if !defined(WINNT)
#ident "$Id: ucrpriv.h,v 1.8 1998/04/22 19:06:57 steve Exp $"
#endif

/*
 * These are difinitions that are likely needed throughout the driver
 * source for the ucr target. The os.h and target.h header files must
 * be included before this.
 */

/* This is the number of buffers allocated per channel (in each
   direction) */
# define CHANNEL_BUFS 4

/* And this the the depth of the value set fifo that sends dma value
   sets to the target. It does not cost much to have a larger number
   here. */
# define FIFO_BUFFER_COUNT 64

/*
 * There are a few parameters that are local to an open file
 * descriptor, such as the current channel. They do not affect the
 * device at all, just the process's view of it. Instances of this are
 * created in the xxopen and released in the xxrelease.
 */
struct ChannelData {
#ifdef OS_CHAN_MEMBERS
      OS_CHAN_MEMBERS
#endif
      unsigned short channel;

	/* These members participate in the flow of data from the host
	   (me) to the target channel. */

      struct shared_buf out_bufs[CHANNEL_BUFS];
      unsigned out_busy, out_nbusy, out_ptr, out_off;
      struct wait_queue*out_sync;
      struct wait_queue*out_status_sync;

	/* ... and these go from the target to me. */

      struct shared_buf in_bufs[CHANNEL_BUFS];
      unsigned in_sz[CHANNEL_BUFS];
      unsigned in_busy, in_nbusy;
      unsigned in_ptr, in_nptr;
      unsigned in_off;
      struct wait_queue*in_sync;

      struct ChannelData *next, *prev;
};

# define STEP_BUF_PTR(ptr) ((ptr) = ((ptr)+1) % CHANNEL_BUFS)

/*
 * These are the parameters common to all the open files on the
 * target. This reflects the state of the target as a whole.
 */
struct Instance {
#if defined(OS_INST_MEMBERS)
      OS_INST_MEMBERS
#endif


      unsigned char number;

	/* These are the PCI bus address of the configuration
	   registers. There may be occasion to access configuration
	   space for the target device. */
      unsigned char bus, dfn;

	/* This is the value passed to writeX by the dev_x
	   functions. The init gets whatever value is needed for write
	   to work (normally the value right out of the BAR
	   register). */
      unsigned long dev;

      struct ChannelData *channels;

	/* The DMA structures are delivered asynchronously. The sender
	   writes them to this fifo, and the target acknowledges them,
	   causing them to be pulled off the front.

	   The sync member is awakened when the structure is
	   delivered, and the status member is awakened when the
	   matching status is delivered. */

      struct {
	    __u32 dma_addr;
	    __u32 dma_count;
	    __u32 bell_mask;
	    struct wait_queue**sync;
      } dma_fifo[FIFO_BUFFER_COUNT];
      unsigned fifo_ptr;
      unsigned fifo_fill;

	/* Manage the state of the supported frames. Each frame has a
	   size and a pagesize, along with a number of pages. the
	   pagesize is generally the size of a host page, but if the
	   O/S chooses to allocate larger physically contiguous
	   chunks, then great. */

      struct {
	    __u32 npages;
	    __u32 pagesize;
	    struct shared_buf*pages;
      } frame[16];

	/* These members manage the sending of DMA value sets to the
	   board. These are synchronization points for the board and
	   the driver. */

      volatile int send_busy_flag;
      struct wait_queue*send_busy_sync;

};

/*
 * This function atomically releases the interrupts (with the mask
 * supplied) and does a sleep on the sync item. This reduces the
 * blocking of global interrupts to the small period during this
 * function, instead of blocking interrupts for long periods of time.
 */
extern void atomic_sleep_on(struct Instance*xsp,
			    unsigned long mask, struct wait_queue**sync);

extern unsigned debug_flag;

extern int wait_for_read_data(struct Instance*xsp, struct ChannelData*xpd);
extern int wait_for_write_space(struct Instance*xsp, struct ChannelData*xpd);

/*
 * These are the ucr functions for performing the various generic
 * operations of the uCR protocol.
 */
extern int  ucr_open(struct Instance*xsp, struct ChannelData*xpd);
extern void ucr_release(struct Instance*xsp, struct ChannelData*xpd);
extern long ucr_read(struct Instance*xsp, struct ChannelData*xpd,
		     char *bytes, unsigned long count, int block_flag);
extern long ucr_write(struct Instance*xsp, struct ChannelData*xpd,
		      const char*bytes, const unsigned long count);
extern int  ucr_ioctl(struct Instance*xsp, struct ChannelData*xpd,
		      unsigned cmd, unsigned long arg);
extern void ucr_irq(struct Instance*xsp);

extern int  ucrx_open(struct Instance*xsp);
extern void ucrx_release(struct Instance*xsp);
extern int  ucrx_ioctl(struct Instance*xsp, unsigned cmd, unsigned long arg);

extern void ucr_pad_channel(struct Instance*xsp, struct ChannelData*xpd);

/*
 * The generic code needs to make frames occasionally, but the task
 * itself is operating system specific. These methods manage the
 * workings of frames.
 */
extern int ucr_make_frame(struct Instance*xsp, unsigned id,
			  unsigned long size);
extern int ucr_free_frame(struct Instance*xsp, unsigned id);

/*
 * These are generic functions that help with the management of the
 * instance structure.
 */
extern void ucr_init_instance(struct Instance*xsp);
extern void ucr_clear_instance(struct Instance*xsp);

/*
 * $Log: ucrpriv.h,v $
 * Revision 1.8  1998/04/22 19:06:57  steve
 *  Some write cancel support.
 *
 * Revision 1.7  1998/04/18 02:43:54  steve
 *  Fix trouble removing device lines at unload.
 *  Add support for read cancellation under NT.
 *  isecons device (nt only) now throws away old
 *  data instead of truncating new.
 *
 * Revision 1.6  1998/03/23 03:30:25  steve
 *  non-blocking reading under NT.
 *
 * Revision 1.5  1998/03/21 21:06:56  steve
 *  Rearrange frame handling code.
 *
 * Revision 1.4  1998/03/11 00:07:17  steve
 *  Support non-blocking input, and add an ioctl
 *  that tests for input data.
 *
 * Revision 1.3  1997/12/02 02:29:04  steve
 *  Add support for ISE board reset.
 *
 * Revision 1.2  1997/11/11 11:19:46  steve
 *  Add support for frames.
 *
 * Revision 1.1  1997/09/11 21:58:36  steve
 *  Port the uCR device driver to Windows NT,
 *  make the configure select the target board
 *  and target host interface headers.
 *
 */


#endif
