/*
 * Copyright (c) 1998 Picture Elements, Inc.
 *    Stephen Williams (steve@picturel.com)
 *
 *    This source code is free software; you can redistribute it
 *    and/or modify it in source code form under the terms of the GNU
 *    General Public License as published by the Free Software
 *    Foundation; either version 2 of the License, or (at your option)
 *    any later version. In order to redistribute the software in
 *    binary form, you will need a Picture Elements Binary Software
 *    License.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *  ---
 *    You should also have recieved a copy of the Picture Elements
 *    Binary Software License offer along with the source. This offer
 *    allows you to obtain the right to redistribute the software in
 *    binary (compiled) form. If you have not received it, contact
 *    Picture Elements, Inc., 777 Panoramic Way, Berkeley, CA 94704.
 */
#if !defined(WINNT)
#ident "$Id: winnt.c,v 1.8 1998/06/10 04:09:12 steve Exp $"
#endif

# include  <tcl.h>
# include  <ucrif.h>
# include  <assert.h>
# include  <stdlib.h>
# include  <malloc.h>
# include  <string.h>
# include  <errno.h>
# include  "priv.h"


int ucrInputProc(ClientData cd, char*buf, int cnt, int*err)
{
      struct ucr_channel_info*info = (struct ucr_channel_info*)cd;
      BOOL rc;
      long out = 0, count = 0;


      if (info->block_mode != TCL_MODE_BLOCKING) {
	    rc = DeviceIoControl(info->fd, UCR_NREAD, 0, 0, &out, sizeof out,
				 &count, 0);
	    assert( rc );
	    assert( count == sizeof out );
	    if (out == 0)
		  return 0;
      }

      rc = ReadFile(info->fd, buf, cnt, &out, 0);
      if (! rc)
	    *err = 1;

      if (out == 0)
	    *err = 2;

      *err = 0;

      return out;
}


int ucrOutputProc(ClientData cd, char*buf, int cnt, int*err)
{
      DWORD junk;
      BOOL rc;
      long out;
      struct ucr_channel_info*info = (struct ucr_channel_info*)cd;

      WriteFile(info->fd, buf, cnt, &out, 0);

      rc = DeviceIoControl(info->fd, UCR_FLUSH, 0, 0, 0, 0, &junk, 0);

      *err = 0;

      return out;
}

struct UcrChanEvent {
      Tcl_Event event;
      struct ucr_channel_info*info;
};

/*
 * Under NT, I am forced to create a custom event handler. This
 * essentially polls the target channel.
 */
static int ucrEventProc(Tcl_Event*ev, int flags)
{
      struct UcrChanEvent*event = (struct UcrChanEvent*)ev;

      if (! (flags & TCL_FILE_EVENTS)) return 0;

      event->info->flags &= ~EVENT_PENDING;
      Tcl_NotifyChannel(event->info->tc, TCL_READABLE);

      return 1;
}

static void ucrCheckProc(ClientData cd, int flags)
{
      BOOL rc;
      int count;
      DWORD size;
      struct UcrChanEvent*ev;
      struct ucr_channel_info*info = (struct ucr_channel_info*)cd;

      if (! (flags & TCL_FILE_EVENTS))
	    return;

      if (info->flags & EVENT_PENDING)
	    return;

      rc = DeviceIoControl(info->fd, UCR_NREAD, 0, 0, &count, sizeof count,
			   &size, 0);
      assert( size == sizeof count );
      if (count == 0)
	    return;

      info->flags |= EVENT_PENDING;
      ev = (struct UcrChanEvent*) ckalloc(sizeof (struct UcrChanEvent));
      ev->event.proc = ucrEventProc;
      ev->info = info;
      Tcl_QueueEvent(&ev->event, TCL_QUEUE_TAIL);
}

static void ucrSetupProc(ClientData cd, int flags)
{
      if (flags & TCL_FILE_EVENTS) {
	    Tcl_Time btime = { 0, 100 };
	    Tcl_SetMaxBlockTime(&btime);
      }
}

void ucrWatchProc(ClientData cd, int mask)
{
      struct ucr_channel_info*info = (struct ucr_channel_info*)cd;

      mask &= TCL_READABLE;
      if (mask) {
	    Tcl_CreateEventSource(ucrSetupProc, ucrCheckProc,
				  (ClientData)info);
      } else {
	    Tcl_DeleteEventSource(ucrSetupProc, ucrCheckProc,
				  (ClientData)info);
      }
}


int ucr_open_channel(struct ucr_channel_info*info, const char*path,
		     Tcl_Interp*interp)
{
      DWORD junk;
      BOOL rc;
      char fmt_path[32];

      sprintf(fmt_path, "\\\\.\\%s", path);

      info->fd = CreateFile(fmt_path, GENERIC_READ|GENERIC_WRITE,
			    FILE_SHARE_READ|FILE_SHARE_WRITE, 0,
			    OPEN_EXISTING,
			    FILE_ATTRIBUTE_NORMAL, 0);
      if (info->fd == INVALID_HANDLE_VALUE) {
	    Tcl_AppendResult(interp, "unable to open ``", path, "''", 0);
	    free(info);
	    return TCL_ERROR;
      }

      rc = DeviceIoControl(info->fd, UCR_CHANNEL, &info->chan,
			   sizeof info->chan,  0, 0, &junk, 0);

      if (!rc) {
	    Tcl_SetResult(interp, "unable to set channel number.", TCL_STATIC);
	    CloseHandle(info->fd);
	    free(info);
	    return TCL_ERROR;
      }

      return TCL_OK;
}

/*
 * For now, this code only supports one frame.
 */

static unsigned long frame_alloc_map = 0;

int ucr_manage_frame(Tcl_Interp*interp, struct ucr_frame_info*finfo,
		     unsigned long frame_size)
{
      DWORD junk;
      BOOL rc;
      struct UcrMmapInfo mapinfo;
      unsigned long frame_parm;

      if (frame_size == 0) {
	    if (finfo->frame_base) {

		  mapinfo.frame_id = finfo->num;
		  mapinfo.off = 0;
		  mapinfo.size = finfo->frame_size;
		  mapinfo.base = finfo->frame_base;

		  rc = DeviceIoControl(finfo->cinfo->fd, UCR_MUNMAP_FRAME,
				       &mapinfo, sizeof mapinfo, 0, 0,
				       &junk, 0);

		  rc = DeviceIoControl(finfo->cinfo->fd,
				       UCR_FREE_FRAME, 0, 0, 0, 0, &junk, 0);

		  finfo->frame_base = 0;
		  finfo->frame_size = 0;
		  frame_alloc_map &= ~(1<<finfo->num);
	    }
	    return TCL_OK;
      }

      assert( interp );

      if (frame_alloc_map & (1<<finfo->num)) {
	    Tcl_SetResult(interp, "frame id is busy.", TCL_STATIC);
	    return TCL_ERROR;
      }

      frame_alloc_map |= 1<<finfo->num;
      frame_parm = (finfo->num<<28) | frame_size;
      rc = DeviceIoControl(finfo->cinfo->fd, UCR_MAKE_FRAME, &frame_parm,
			   sizeof frame_parm, &frame_parm,
			   sizeof frame_parm, &junk, 0);
      if (rc == FALSE) {
	    Tcl_SetResult(interp, "Error making frame id.", TCL_STATIC);
	    return TCL_ERROR;
      }

      finfo->frame_size = frame_parm & 0x0fffffff;

      mapinfo.frame_id = finfo->num;
      mapinfo.off = 0;
      mapinfo.size = frame_size;
      mapinfo.base = 0;

      rc = DeviceIoControl(finfo->cinfo->fd, UCR_MMAP_FRAME, &mapinfo,
			   sizeof mapinfo, &mapinfo, sizeof mapinfo,
			   &junk, 0);

      finfo->frame_size = mapinfo.size;
      finfo->frame_base = mapinfo.base;


      return TCL_OK;
}

/*
 * $Log: winnt.c,v $
 * Revision 1.8  1998/06/10 04:09:12  steve
 *  redo the frame interface.
 *
 * Revision 1.7  1998/06/06 05:40:04  steve
 *  Linux/alpha 2.1.100 port.
 *
 * Revision 1.6  1998/05/29 18:40:47  steve
 *  Different error codes for read error.
 *
 * Revision 1.5  1998/04/30 01:56:47  steve
 *  Get rid of overlapped I/O, for now.
 *
 * Revision 1.4  1998/04/22 19:07:58  steve
 *  Some excess cruft.
 *
 * Revision 1.3  1998/03/23 19:56:09  steve
 *  Support for frames under NT,
 *  and non-blocking I/O under tcl/nt
 *
 * Revision 1.2  1998/03/12 23:00:25  steve
 *  Add frame commands.
 *
 * Revision 1.1  1998/03/12 21:32:29  steve
 *  divide the NT and linux implementataion up.
 *
 */

